# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class TargetSchema(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, connection_types: List[str]=None, default: str=None, name: str=None, required: bool=None):  # noqa: E501
        """TargetSchema - a model defined in Swagger

        :param connection_types: The connection_types of this TargetSchema.  # noqa: E501
        :type connection_types: List[str]
        :param default: The default of this TargetSchema.  # noqa: E501
        :type default: str
        :param name: The name of this TargetSchema.  # noqa: E501
        :type name: str
        :param required: The required of this TargetSchema.  # noqa: E501
        :type required: bool
        """
        self.swagger_types = {
            'connection_types': List[str],
            'default': str,
            'name': str,
            'required': bool
        }

        self.attribute_map = {
            'connection_types': 'connectionTypes',
            'default': 'default',
            'name': 'name',
            'required': 'required'
        }

        self._connection_types = connection_types
        self._default = default
        self._name = name
        self._required = required

    @classmethod
    def from_dict(cls, dikt) -> 'TargetSchema':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The TargetSchema of this TargetSchema.  # noqa: E501
        :rtype: TargetSchema
        """
        return util.deserialize_model(dikt, cls)

    @property
    def connection_types(self) -> List[str]:
        """Gets the connection_types of this TargetSchema.


        :return: The connection_types of this TargetSchema.
        :rtype: List[str]
        """
        return self._connection_types

    @connection_types.setter
    def connection_types(self, connection_types: List[str]):
        """Sets the connection_types of this TargetSchema.


        :param connection_types: The connection_types of this TargetSchema.
        :type connection_types: List[str]
        """

        self._connection_types = connection_types

    @property
    def default(self) -> str:
        """Gets the default of this TargetSchema.


        :return: The default of this TargetSchema.
        :rtype: str
        """
        return self._default

    @default.setter
    def default(self, default: str):
        """Sets the default of this TargetSchema.


        :param default: The default of this TargetSchema.
        :type default: str
        """

        self._default = default

    @property
    def name(self) -> str:
        """Gets the name of this TargetSchema.


        :return: The name of this TargetSchema.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """Sets the name of this TargetSchema.


        :param name: The name of this TargetSchema.
        :type name: str
        """

        self._name = name

    @property
    def required(self) -> bool:
        """Gets the required of this TargetSchema.


        :return: The required of this TargetSchema.
        :rtype: bool
        """
        return self._required

    @required.setter
    def required(self, required: bool):
        """Sets the required of this TargetSchema.


        :param required: The required of this TargetSchema.
        :type required: bool
        """

        self._required = required



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.external_url import ExternalUrl  # noqa: F401,E501
from odahuflow.sdk.models import util


class CommonConfiguration(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, external_urls: List[ExternalUrl]=None):  # noqa: E501
        """CommonConfiguration - a model defined in Swagger

        :param external_urls: The external_urls of this CommonConfiguration.  # noqa: E501
        :type external_urls: List[ExternalUrl]
        """
        self.swagger_types = {
            'external_urls': List[ExternalUrl]
        }

        self.attribute_map = {
            'external_urls': 'externalUrls'
        }

        self._external_urls = external_urls

    @classmethod
    def from_dict(cls, dikt) -> 'CommonConfiguration':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The CommonConfiguration of this CommonConfiguration.  # noqa: E501
        :rtype: CommonConfiguration
        """
        return util.deserialize_model(dikt, cls)

    @property
    def external_urls(self) -> List[ExternalUrl]:
        """Gets the external_urls of this CommonConfiguration.

        The collection of external urls, for example: metrics, edge, service catalog and so on  # noqa: E501

        :return: The external_urls of this CommonConfiguration.
        :rtype: List[ExternalUrl]
        """
        return self._external_urls

    @external_urls.setter
    def external_urls(self, external_urls: List[ExternalUrl]):
        """Sets the external_urls of this CommonConfiguration.

        The collection of external urls, for example: metrics, edge, service catalog and so on  # noqa: E501

        :param external_urls: The external_urls of this CommonConfiguration.
        :type external_urls: List[ExternalUrl]
        """

        self._external_urls = external_urls



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.resource_list import ResourceList  # noqa: F401,E501
from odahuflow.sdk.models import util


class ResourceRequirements(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, limits: ResourceList=None, requests: ResourceList=None):  # noqa: E501
        """ResourceRequirements - a model defined in Swagger

        :param limits: The limits of this ResourceRequirements.  # noqa: E501
        :type limits: ResourceList
        :param requests: The requests of this ResourceRequirements.  # noqa: E501
        :type requests: ResourceList
        """
        self.swagger_types = {
            'limits': ResourceList,
            'requests': ResourceList
        }

        self.attribute_map = {
            'limits': 'limits',
            'requests': 'requests'
        }

        self._limits = limits
        self._requests = requests

    @classmethod
    def from_dict(cls, dikt) -> 'ResourceRequirements':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ResourceRequirements of this ResourceRequirements.  # noqa: E501
        :rtype: ResourceRequirements
        """
        return util.deserialize_model(dikt, cls)

    @property
    def limits(self) -> ResourceList:
        """Gets the limits of this ResourceRequirements.

        Limits describes the maximum amount of compute resources allowed.  # noqa: E501

        :return: The limits of this ResourceRequirements.
        :rtype: ResourceList
        """
        return self._limits

    @limits.setter
    def limits(self, limits: ResourceList):
        """Sets the limits of this ResourceRequirements.

        Limits describes the maximum amount of compute resources allowed.  # noqa: E501

        :param limits: The limits of this ResourceRequirements.
        :type limits: ResourceList
        """

        self._limits = limits

    @property
    def requests(self) -> ResourceList:
        """Gets the requests of this ResourceRequirements.

        Requests describes the minimum amount of compute resources required.  # noqa: E501

        :return: The requests of this ResourceRequirements.
        :rtype: ResourceList
        """
        return self._requests

    @requests.setter
    def requests(self, requests: ResourceList):
        """Sets the requests of this ResourceRequirements.

        Requests describes the minimum amount of compute resources required.  # noqa: E501

        :param requests: The requests of this ResourceRequirements.
        :type requests: ResourceList
        """

        self._requests = requests



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class ModelIdentity(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, artifact_name_template: str=None, name: str=None, version: str=None):  # noqa: E501
        """ModelIdentity - a model defined in Swagger

        :param artifact_name_template: The artifact_name_template of this ModelIdentity.  # noqa: E501
        :type artifact_name_template: str
        :param name: The name of this ModelIdentity.  # noqa: E501
        :type name: str
        :param version: The version of this ModelIdentity.  # noqa: E501
        :type version: str
        """
        self.swagger_types = {
            'artifact_name_template': str,
            'name': str,
            'version': str
        }

        self.attribute_map = {
            'artifact_name_template': 'artifactNameTemplate',
            'name': 'name',
            'version': 'version'
        }

        self._artifact_name_template = artifact_name_template
        self._name = name
        self._version = version

    @classmethod
    def from_dict(cls, dikt) -> 'ModelIdentity':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ModelIdentity of this ModelIdentity.  # noqa: E501
        :rtype: ModelIdentity
        """
        return util.deserialize_model(dikt, cls)

    @property
    def artifact_name_template(self) -> str:
        """Gets the artifact_name_template of this ModelIdentity.

        Template of output artifact name  # noqa: E501

        :return: The artifact_name_template of this ModelIdentity.
        :rtype: str
        """
        return self._artifact_name_template

    @artifact_name_template.setter
    def artifact_name_template(self, artifact_name_template: str):
        """Sets the artifact_name_template of this ModelIdentity.

        Template of output artifact name  # noqa: E501

        :param artifact_name_template: The artifact_name_template of this ModelIdentity.
        :type artifact_name_template: str
        """

        self._artifact_name_template = artifact_name_template

    @property
    def name(self) -> str:
        """Gets the name of this ModelIdentity.

        Model name  # noqa: E501

        :return: The name of this ModelIdentity.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """Sets the name of this ModelIdentity.

        Model name  # noqa: E501

        :param name: The name of this ModelIdentity.
        :type name: str
        """

        self._name = name

    @property
    def version(self) -> str:
        """Gets the version of this ModelIdentity.

        Model version  # noqa: E501

        :return: The version of this ModelIdentity.
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version: str):
        """Sets the version of this ModelIdentity.

        Model version  # noqa: E501

        :param version: The version of this ModelIdentity.
        :type version: str
        """

        self._version = version



import typing

import pydantic


class OdahuflowProjectManifestBinaries(pydantic.BaseModel):
    """
    Odahuflow Project Manifest's Binaries description
    """

    type: str
    dependencies: str
    conda_path: typing.Optional[str]


class OdahuflowProjectManifestModel(pydantic.BaseModel):
    """
    Odahuflow Project Manifest's Model description
    """

    name: str
    version: str
    workDir: str
    entrypoint: str


class OdahuflowProjectManifestToolchain(pydantic.BaseModel):
    """
    Odahuflow Project Manifest's Toolchain description
    """

    name: str
    version: str


class OdahuflowProjectManifest(pydantic.BaseModel):
    """
    Odahuflow Project Manifest description class
    """

    binaries: OdahuflowProjectManifestBinaries
    model: typing.Optional[OdahuflowProjectManifestModel]
    odahuflowVersion: typing.Optional[str]
    toolchain: typing.Optional[OdahuflowProjectManifestToolchain]



#
#    Copyright 2017 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
API client
"""
import logging
import typing

from odahuflow.sdk.clients.api import RemoteAPIClient, AsyncRemoteAPIClient
from odahuflow.sdk.definitions import CONNECTION_URL
from odahuflow.sdk.models import Connection

LOGGER = logging.getLogger(__name__)


class ConnectionClient(RemoteAPIClient):
    """
    HTTP connection client
    """

    def get(self, conn_id: str) -> Connection:
        """
        Get Connection from API server

        :param conn_id: Connection ID
        :return: Connection
        """
        return Connection.from_dict(self.query(f'{CONNECTION_URL}/{conn_id}'))

    def get_decrypted(self, conn_id: str) -> Connection:
        """
        Get decrypted connection from API server

        :param conn_id: Connection ID
        :return: Connection
        """
        return Connection.from_dict(self.query(
            f'{CONNECTION_URL}/{conn_id}/decrypted'
        ))

    def get_all(self) -> typing.List[Connection]:
        """
        Get all Connections from API server

        :return: all Connections
        """
        return [Connection.from_dict(conn) for conn in self.query(CONNECTION_URL)]

    def create(self, conn: Connection) -> Connection:
        """
        Create Connection

        :param conn: Connection
        :return Message from API server
        """
        return Connection.from_dict(self.query(CONNECTION_URL, action='POST', payload=conn.to_dict()))

    def edit(self, conn: Connection) -> Connection:
        """
        Edit Connection

        :param conn: Connection
        :return Message from API server
        """
        return Connection.from_dict(self.query(CONNECTION_URL, action='PUT', payload=conn.to_dict()))

    def delete(self, name: str) -> str:
        """
        Delete Connections

        :param name: Name of a Connection
        :return Message from API server
        """
        return self.query(f'{CONNECTION_URL}/{name}', action='DELETE')['message']


class AsyncConnectionClient(AsyncRemoteAPIClient):
    """
    HTTP connection async client
    """

    async def get(self, conn_id: str) -> Connection:
        """
        Get Connection from API server

        :param conn_id: Connection ID
        :return: Connection
        """
        return Connection.from_dict(await self.query(f'{CONNECTION_URL}/{conn_id}'))

    async def get_decrypted(self, conn_id: str) -> Connection:
        """
        Get decrypted connection from API server

        :param conn_id: Connection ID
        :return: Connection
        """
        return Connection.from_dict(await self.query(
            f'{CONNECTION_URL}/{conn_id}/decrypted'
        ))

    async def get_all(self) -> typing.List[Connection]:
        """
        Get all Connections from API server

        :return: all Connections
        """
        return [Connection.from_dict(conn) for conn in await self.query(CONNECTION_URL)]

    async def create(self, conn: Connection) -> Connection:
        """
        Create Connection

        :param conn: Connection
        :return Message from API server
        """
        return Connection.from_dict(await self.query(CONNECTION_URL, action='POST', payload=conn.to_dict()))

    async def edit(self, conn: Connection) -> Connection:
        """
        Edit Connection

        :param conn: Connection
        :return Message from API server
        """
        return Connection.from_dict(await self.query(CONNECTION_URL, action='PUT', payload=conn.to_dict()))

    async def delete(self, name: str) -> str:
        """
        Delete Connections

        :param name: Name of a Connection
        :return Message from API server
        """
        return (await self.query(f'{CONNECTION_URL}/{name}', action='DELETE'))['message']



# coding: utf-8

# flake8: noqa
from __future__ import absolute_import
# import models into model package
from odahuflow.sdk.models.common_configuration import CommonConfiguration
from odahuflow.sdk.models.configuration import Configuration
from odahuflow.sdk.models.connection import Connection
from odahuflow.sdk.models.connection_spec import ConnectionSpec
from odahuflow.sdk.models.connection_status import ConnectionStatus
from odahuflow.sdk.models.data_binding_dir import DataBindingDir
from odahuflow.sdk.models.environment_variable import EnvironmentVariable
from odahuflow.sdk.models.external_url import ExternalUrl
from odahuflow.sdk.models.feedback_model_feedback_request import FeedbackModelFeedbackRequest
from odahuflow.sdk.models.feedback_model_feedback_response import FeedbackModelFeedbackResponse
from odahuflow.sdk.models.http_result import HTTPResult
from odahuflow.sdk.models.input_data_binding_dir import InputDataBindingDir
from odahuflow.sdk.models.json_schema import JsonSchema
from odahuflow.sdk.models.k8s_packager import K8sPackager
from odahuflow.sdk.models.k8s_trainer import K8sTrainer
from odahuflow.sdk.models.model_deployment import ModelDeployment
from odahuflow.sdk.models.model_deployment_spec import ModelDeploymentSpec
from odahuflow.sdk.models.model_deployment_status import ModelDeploymentStatus
from odahuflow.sdk.models.model_deployment_target import ModelDeploymentTarget
from odahuflow.sdk.models.model_identity import ModelIdentity
from odahuflow.sdk.models.model_packaging import ModelPackaging
from odahuflow.sdk.models.model_packaging_result import ModelPackagingResult
from odahuflow.sdk.models.model_packaging_spec import ModelPackagingSpec
from odahuflow.sdk.models.model_packaging_status import ModelPackagingStatus
from odahuflow.sdk.models.model_property import ModelProperty
from odahuflow.sdk.models.model_route import ModelRoute
from odahuflow.sdk.models.model_route_spec import ModelRouteSpec
from odahuflow.sdk.models.model_route_status import ModelRouteStatus
from odahuflow.sdk.models.model_training import ModelTraining
from odahuflow.sdk.models.model_training_spec import ModelTrainingSpec
from odahuflow.sdk.models.model_training_status import ModelTrainingStatus
from odahuflow.sdk.models.packager_target import PackagerTarget
from odahuflow.sdk.models.packaging_integration import PackagingIntegration
from odahuflow.sdk.models.packaging_integration_spec import PackagingIntegrationSpec
from odahuflow.sdk.models.packaging_integration_status import PackagingIntegrationStatus
from odahuflow.sdk.models.parameter import Parameter
from odahuflow.sdk.models.resource_list import ResourceList
from odahuflow.sdk.models.resource_requirements import ResourceRequirements
from odahuflow.sdk.models.schema import Schema
from odahuflow.sdk.models.target import Target
from odahuflow.sdk.models.target_schema import TargetSchema
from odahuflow.sdk.models.toolchain_integration import ToolchainIntegration
from odahuflow.sdk.models.toolchain_integration_spec import ToolchainIntegrationSpec
from odahuflow.sdk.models.toolchain_integration_status import ToolchainIntegrationStatus
from odahuflow.sdk.models.training_configuration import TrainingConfiguration
from odahuflow.sdk.models.training_result import TrainingResult



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.connection import Connection  # noqa: F401,E501
from odahuflow.sdk.models.model_packaging import ModelPackaging  # noqa: F401,E501
from odahuflow.sdk.models.packager_target import PackagerTarget  # noqa: F401,E501
from odahuflow.sdk.models.packaging_integration import PackagingIntegration  # noqa: F401,E501
from odahuflow.sdk.models import util


class K8sPackager(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, model_holder: Connection=None, model_packaging: ModelPackaging=None, packaging_integration: PackagingIntegration=None, targets: List[PackagerTarget]=None, training_zip_name: str=None):  # noqa: E501
        """K8sPackager - a model defined in Swagger

        :param model_holder: The model_holder of this K8sPackager.  # noqa: E501
        :type model_holder: Connection
        :param model_packaging: The model_packaging of this K8sPackager.  # noqa: E501
        :type model_packaging: ModelPackaging
        :param packaging_integration: The packaging_integration of this K8sPackager.  # noqa: E501
        :type packaging_integration: PackagingIntegration
        :param targets: The targets of this K8sPackager.  # noqa: E501
        :type targets: List[PackagerTarget]
        :param training_zip_name: The training_zip_name of this K8sPackager.  # noqa: E501
        :type training_zip_name: str
        """
        self.swagger_types = {
            'model_holder': Connection,
            'model_packaging': ModelPackaging,
            'packaging_integration': PackagingIntegration,
            'targets': List[PackagerTarget],
            'training_zip_name': str
        }

        self.attribute_map = {
            'model_holder': 'modelHolder',
            'model_packaging': 'modelPackaging',
            'packaging_integration': 'packagingIntegration',
            'targets': 'targets',
            'training_zip_name': 'trainingZipName'
        }

        self._model_holder = model_holder
        self._model_packaging = model_packaging
        self._packaging_integration = packaging_integration
        self._targets = targets
        self._training_zip_name = training_zip_name

    @classmethod
    def from_dict(cls, dikt) -> 'K8sPackager':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The K8sPackager of this K8sPackager.  # noqa: E501
        :rtype: K8sPackager
        """
        return util.deserialize_model(dikt, cls)

    @property
    def model_holder(self) -> Connection:
        """Gets the model_holder of this K8sPackager.

        Connection where a trained model artifact is stored  # noqa: E501

        :return: The model_holder of this K8sPackager.
        :rtype: Connection
        """
        return self._model_holder

    @model_holder.setter
    def model_holder(self, model_holder: Connection):
        """Sets the model_holder of this K8sPackager.

        Connection where a trained model artifact is stored  # noqa: E501

        :param model_holder: The model_holder of this K8sPackager.
        :type model_holder: Connection
        """

        self._model_holder = model_holder

    @property
    def model_packaging(self) -> ModelPackaging:
        """Gets the model_packaging of this K8sPackager.

        Model Packaging  # noqa: E501

        :return: The model_packaging of this K8sPackager.
        :rtype: ModelPackaging
        """
        return self._model_packaging

    @model_packaging.setter
    def model_packaging(self, model_packaging: ModelPackaging):
        """Sets the model_packaging of this K8sPackager.

        Model Packaging  # noqa: E501

        :param model_packaging: The model_packaging of this K8sPackager.
        :type model_packaging: ModelPackaging
        """

        self._model_packaging = model_packaging

    @property
    def packaging_integration(self) -> PackagingIntegration:
        """Gets the packaging_integration of this K8sPackager.

        Packaging integration  # noqa: E501

        :return: The packaging_integration of this K8sPackager.
        :rtype: PackagingIntegration
        """
        return self._packaging_integration

    @packaging_integration.setter
    def packaging_integration(self, packaging_integration: PackagingIntegration):
        """Sets the packaging_integration of this K8sPackager.

        Packaging integration  # noqa: E501

        :param packaging_integration: The packaging_integration of this K8sPackager.
        :type packaging_integration: PackagingIntegration
        """

        self._packaging_integration = packaging_integration

    @property
    def targets(self) -> List[PackagerTarget]:
        """Gets the targets of this K8sPackager.

        List of targets with appropriate connections  # noqa: E501

        :return: The targets of this K8sPackager.
        :rtype: List[PackagerTarget]
        """
        return self._targets

    @targets.setter
    def targets(self, targets: List[PackagerTarget]):
        """Sets the targets of this K8sPackager.

        List of targets with appropriate connections  # noqa: E501

        :param targets: The targets of this K8sPackager.
        :type targets: List[PackagerTarget]
        """

        self._targets = targets

    @property
    def training_zip_name(self) -> str:
        """Gets the training_zip_name of this K8sPackager.

        Name of trained model artifact name  # noqa: E501

        :return: The training_zip_name of this K8sPackager.
        :rtype: str
        """
        return self._training_zip_name

    @training_zip_name.setter
    def training_zip_name(self, training_zip_name: str):
        """Sets the training_zip_name of this K8sPackager.

        Name of trained model artifact name  # noqa: E501

        :param training_zip_name: The training_zip_name of this K8sPackager.
        :type training_zip_name: str
        """

        self._training_zip_name = training_zip_name



import json
import logging
from os.path import join
from typing import Dict, Any

import docker
from docker.models.containers import Container
from docker.types import Mount
from odahuflow.sdk import config
from odahuflow.sdk.local.docker_utils import stream_container_logs, \
    convert_labels_to_filter, cleanup_docker_containers, PACKAGING_DOCKER_LABELS, raise_error_if_container_failed
from odahuflow.sdk.models import K8sPackager

PACKAGER_CONF_FILE_PATH = 'mp.json'
PACKAGER_RESULT_FILE_PATH = 'result.json'
ARTIFACT_PATH = '/trained_artifact'

LOGGER = logging.getLogger(__name__)


def create_mp_config_file(config_dir: str, packager: K8sPackager) -> None:
    with open(join(config_dir, PACKAGER_CONF_FILE_PATH), 'w') as f:
        packager_dict = packager.to_dict()
        json.dump(packager_dict, f)

        LOGGER.debug(f"Saved the packaging configuration:\n{json.dumps(packager_dict, indent=2)}")


def read_mp_result_file(config_dir: str) -> Dict[str, Any]:
    with open(join(config_dir, PACKAGER_RESULT_FILE_PATH)) as f:
        return json.load(f)


def start_package(packager: K8sPackager, artifact_path: str) -> Dict[str, Any]:
    if not artifact_path:
        artifact_path = join(
            config.LOCAL_MODEL_OUTPUT_DIR,
            packager.model_packaging.spec.artifact_name
        )

    create_mp_config_file(artifact_path, packager)

    client = docker.from_env()
    container: Container = client.containers.run(
        packager.model_packaging.spec.image or packager.packaging_integration.spec.default_image,
        stderr=True,
        stdout=True,
        working_dir=ARTIFACT_PATH,
        command=[
            packager.packaging_integration.spec.entrypoint,
            ARTIFACT_PATH,
            join(ARTIFACT_PATH, PACKAGER_CONF_FILE_PATH),
        ],
        mounts=[
            Mount(ARTIFACT_PATH, artifact_path, type="bind"),
            Mount("/var/run/docker.sock", "/var/run/docker.sock", type="bind")
        ],
        detach=True,
        labels=PACKAGING_DOCKER_LABELS,
    )

    container_info = client.api.inspect_container(container.id)
    LOGGER.debug(f'Container info:\n{json.dumps(container_info, indent=2)}')

    print(f'Packaging docker image {container.id} has started. Stream logs:')
    stream_container_logs(container)

    raise_error_if_container_failed(container.id)

    return read_mp_result_file(artifact_path)


def cleanup_packaging_docker_containers():
    cleanup_docker_containers(convert_labels_to_filter(PACKAGING_DOCKER_LABELS))



#
#    Copyright 2019 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
odahuflow env names
"""
import configparser
import logging
import os
from pathlib import Path

# Get list of all variables
ALL_VARIABLES = {}

_LOGGER = logging.getLogger()

_INI_FILE_TRIED_TO_BE_LOADED = False
_INI_FILE_CONTENT: configparser.ConfigParser = None
_INI_FILE_DEFAULT_CONFIG_PATH = Path.home().joinpath('.odahuflow/config')
_DEFAULT_INI_SECTION = 'general'
MODEL_JWT_TOKEN_SECTION = 'model_tokens'


def reset_context():
    """
    Reset configuration context

    :return: None
    """
    global _INI_FILE_TRIED_TO_BE_LOADED
    global _INI_FILE_CONTENT

    _INI_FILE_TRIED_TO_BE_LOADED = False
    _INI_FILE_CONTENT = None


def get_config_file_path():
    """
    Return the config path.
    ODAHUFLOW_CONFIG environment can override path value

    :return: Path -- config path
    """
    config_path_from_env = os.getenv('ODAHUFLOW_CONFIG')

    return Path(config_path_from_env) if config_path_from_env else _INI_FILE_DEFAULT_CONFIG_PATH


def _load_config_file():
    """
    Load configuration file if it has not been loaded. Update _INI_FILE_TRIED_TO_BE_LOADED, _INI_FILE_CONTENT

    :return: None
    """
    global _INI_FILE_TRIED_TO_BE_LOADED

    if _INI_FILE_TRIED_TO_BE_LOADED:
        return

    config_path = get_config_file_path()
    _INI_FILE_TRIED_TO_BE_LOADED = True

    _LOGGER.debug('Trying to load configuration file {}'.format(config_path))

    try:
        if config_path.exists():
            config = configparser.ConfigParser()
            config.read(str(config_path))

            global _INI_FILE_CONTENT
            _INI_FILE_CONTENT = config

            _LOGGER.debug('Configuration file {} has been loaded'.format(config_path))
        else:
            _LOGGER.debug('Cannot find configuration file {}'.format(config_path))
    except Exception as exc:
        _LOGGER.exception('Cannot read config file {}'.format(config_path), exc_info=exc)


def get_config_file_section(section=_DEFAULT_INI_SECTION, silent=False):
    """
    Get section from config file

    :param section: (Optional) name of section
    :type section: str
    :param silent: (Optional) ignore if there is no file
    :type silent: bool
    :return: dict[str, str] -- values from section
    """
    _load_config_file()
    if not _INI_FILE_CONTENT:
        if silent:
            return dict()
        else:
            raise Exception('Configuration file cannot be loaded')

    if not _INI_FILE_CONTENT.has_section(section):
        return {}

    return dict(_INI_FILE_CONTENT[section])


def get_config_file_variable(variable, section=_DEFAULT_INI_SECTION):
    """
    Get variable by name from specific (or default) section

    :param variable: Name of variable
    :type variable: str
    :param section: (Optional) name of section
    :type section: str
    :return: str or None -- value
    """
    if not variable:
        return None

    _load_config_file()
    if not _INI_FILE_CONTENT:
        return None

    return _INI_FILE_CONTENT.get(section, variable, fallback=None)


def update_config_file(section=_DEFAULT_INI_SECTION, **new_values):
    """
    Update config file with new values

    :param section: (Optional) name of section to update
    :type section: str
    :param new_values: new values
    :type new_values: dict[str, typing.Optional[str]]
    :return: None
    """
    global _INI_FILE_TRIED_TO_BE_LOADED
    global _INI_FILE_CONTENT

    _load_config_file()
    config_path = get_config_file_path()

    content = _INI_FILE_CONTENT if _INI_FILE_CONTENT else configparser.ConfigParser()

    config_path.parent.mkdir(mode=0o775, parents=True, exist_ok=True)
    config_path.touch(mode=0o600, exist_ok=True)

    if not content.has_section(section):
        content.add_section(section)

    for key, value in new_values.items():
        if value:
            content.set(section, key, value)
        else:
            if section in content and key in content[section]:
                del content[section][key]

    with config_path.open('w') as config_file:
        content.write(config_file)

    _INI_FILE_TRIED_TO_BE_LOADED = True
    _INI_FILE_CONTENT = content

    reinitialize_variables()

    _LOGGER.debug('Configuration file {} has been updated'.format(config_path))


def _load_variable(name, cast_type=None, configurable_manually=True):
    """
    Load variable from config file, env. Cast it to desired type.

    :param name: name of variable
    :type name: str
    :param cast_type: (Optional) function to cast
    :type cast_type: Callable[[str], any]
    :param configurable_manually: (Optional) could this variable be configured manually or not
    :type configurable_manually: bool
    :return: Any -- variable value
    """
    value = None

    # 1st level - configuration file
    if configurable_manually:
        conf_value = get_config_file_variable(name)
        if conf_value:
            value = conf_value

    # 2nd level - env. variable
    env_value = os.environ.get(name)
    if env_value:
        value = env_value

    return cast_type(value) if value is not None else None


class ConfigVariableInformation:
    """
    Object holds information about variable (name, default value, casting function, description and etc.)
    """

    def __init__(self, name, default, cast_func, description, configurable_manually):
        """
        Build information about variable

        :param name: name of variable
        :type name: str
        :param default: default value
        :type default: Any
        :param cast_func: cast function
        :type cast_func: Callable[[str], any]
        :param description: description
        :type description: str
        :param configurable_manually: is configurable manually
        :type configurable_manually: bool
        """
        self._name = name
        self._default = default
        self._cast_func = cast_func
        self._description = description
        self._configurable_manually = configurable_manually

    @property
    def name(self):
        """
        Get name of variable

        :return: str -- name
        """
        return self._name

    @property
    def default(self):
        """
        Get default variable value

        :return: Any -- default value
        """
        return self._default

    @property
    def cast_func(self):
        """
        Get cast function (from string to desired type)

        :return: Callable[[str], any] -- casting function
        """
        return self._cast_func

    @property
    def description(self):
        """
        Get human-readable description

        :return: str -- description
        """
        return self._description

    @property
    def configurable_manually(self):
        """
        Is this variable human-configurabe?

        :return: bool -- is human configurable
        """
        return self._configurable_manually


def cast_bool(value):
    """
    Convert string to bool

    :param value: string or bool
    :type value: str or bool
    :return: bool
    """
    if value is None:
        return None

    if isinstance(value, bool):
        return value

    return value.lower() in ['true', '1', 't', 'y', 'yes']


def reinitialize_variables():
    """
    Reinitialize variables due to new ENV variables

    :return: None
    """
    for value_information in ALL_VARIABLES.values():
        explicit_value = _load_variable(value_information.name,
                                        value_information.cast_func,
                                        value_information.configurable_manually)
        value = explicit_value if explicit_value is not None else value_information.default

        globals()[value_information.name] = value


class ConfigVariableDeclaration:
    """
    Class that builds declaration of variable (and returns it's value as an instance)
    """

    def __new__(cls, name, default=None, cast_func=str, description=None, configurable_manually=True):
        """
        Create new instance

        :param name: name of variable
        :type name: str
        :param default: (Optional) default variable value [will not be passed to cast_func]
        :type default: Any
        :param cast_func: (Optional) cast function for variable value
        :type cast_func: Callable[[str], any]
        :param description: (Optional) human-readable variable description
        :type description: str
        :param configurable_manually: (Optional) can be modified by config file or CLI
        :type configurable_manually: bool
        :return: Any -- default or explicit value
        """
        information = ConfigVariableInformation(name, default, cast_func, description, configurable_manually)

        explicit_value = _load_variable(name, cast_func, configurable_manually)
        value = explicit_value if explicit_value is not None else default
        ALL_VARIABLES[information.name] = information
        return value


# Verbose tracing
DEBUG = ConfigVariableDeclaration('DEBUG', False, cast_bool,
                                  'Enable verbose program output',
                                  True)

# Model building information
BUILD_NUMBER = ConfigVariableDeclaration('BUILD_NUMBER', 0, int,
                                         'Number of current build. It is used in model saving (during training)',
                                         False)
BUILD_ID = ConfigVariableDeclaration('BUILD_ID', None, str,
                                     'ID of current build. It is used in model saving (during training)',
                                     False)
BUILD_URL = ConfigVariableDeclaration('BUILD_URL', None, str,
                                      'URL of current build. It is used in model saving (during training)',
                                      False)
BUILD_TAG = ConfigVariableDeclaration('BUILD_TAG', None, str,
                                      'TAG of current build. It is used in model saving (during training)',
                                      False)

GIT_COMMIT = ConfigVariableDeclaration('GIT_COMMIT', None, str,
                                       'Git commit of source code. It is used in model saving (during training)',
                                       False)
GIT_BRANCH = ConfigVariableDeclaration('GIT_BRANCH', None, str,
                                       'Git branch of source code. It is used in model saving (during training)',
                                       False)

JOB_NAME = ConfigVariableDeclaration('JOB_NAME', None, str,
                                     'Name of train job [Jenkins]. It is used in model saving (during training)',
                                     False)
NODE_NAME = ConfigVariableDeclaration('NODE_NAME', None, str,
                                      'Name of node [Jenkins]. It is used in model saving (during training)',
                                      False)

# Model invocation testing
MODEL_SERVER_URL = ConfigVariableDeclaration('MODEL_SERVER_URL', '', str, 'Default url of model server', True)

MODEL_PREFIX_URL = ConfigVariableDeclaration('MODEL_PREFIX_URL', '', str, 'Default prefix url of model server', True)

MODEL_HOST = ConfigVariableDeclaration('MODEL_HOST', '', str, 'Default host of model server', True)

MODEL_DEPLOYMENT_NAME = ConfigVariableDeclaration('MODEL_DEPLOYMENT_NAME', '', str, 'Model deployment name', True)

MODEL_ROUTE_NAME = ConfigVariableDeclaration('MODEL_ROUTE_NAME', '', str, 'Model deployment name', True)

MODEL_JWT_TOKEN = ConfigVariableDeclaration('MODEL_JWT_TOKEN', '', str,
                                            'Model jwt token for access to the model', True)

# Building and serving
MODEL_FILE = ConfigVariableDeclaration('MODEL_FILE', None, str,
                                       'Binary model file location. It is used in model building',
                                       False)

# Training metrics
MODEL_CLUSTER_TRAIN_METRICS_ENABLED = ConfigVariableDeclaration(
    'MODEL_CLUSTER_TRAIN_METRICS_ENABLED', False, cast_bool, 'Send model metrics on train', False
)
MODEL_LOCAL_METRIC_STORE = ConfigVariableDeclaration(
    'MODEL_LOCAL_METRIC_STORE', '.odahuflow/build_metric_store.json', str,
    'File name where model build metric are saved', True
)
METRICS_HOST = ConfigVariableDeclaration('METRICS_HOST', 'graphite', str,
                                         'Host that gets train metrics. It is used during model training', False)
METRICS_PORT = ConfigVariableDeclaration('METRICS_PORT', 9125, int,
                                         'Port that gets train metrics. It is used during model training', False)

# API for models
ODAHUFLOW_ADDR = ConfigVariableDeclaration('ODAHUFLOW_ADDR', '0.0.0.0', str,
                                        'Address to listen Odahuflow model endpoint',
                                        False)
ODAHUFLOW_PORT = ConfigVariableDeclaration('ODAHUFLOW_PORT', 5000, str,
                                        'Port to listen Odahuflow model endpoint',
                                        False)

# API for services
ODAHUFLOW_API_ADDR = ConfigVariableDeclaration('ODAHUFLOW_API_ADDR', '0.0.0.0', str,
                                            'Address to listen Odahuflow service endpoint',
                                            False)
ODAHUFLOW_API_PORT = ConfigVariableDeclaration('ODAHUFLOW_API_PORT', 5001, int,
                                            'Port to listen Odahuflow service endpoint',
                                            False)

# API endpoint
API_URL = ConfigVariableDeclaration('API_URL', 'http://localhost:5000', str,
                                    'URL of API server',
                                    True)
API_TOKEN = ConfigVariableDeclaration('API_TOKEN', None, str,
                                      'Token for API server authorisation',
                                      True)
API_REFRESH_TOKEN = ConfigVariableDeclaration('API_REFRESH_TOKEN', None, str,
                                              'Refresh token',
                                              True)
API_ACCESS_TOKEN = ConfigVariableDeclaration('API_ACCESS_TOKEN', None, str,
                                             'Access token',
                                             True)
API_ISSUING_URL = ConfigVariableDeclaration('API_ISSUING_URL', None, str,
                                            'URL for refreshing and issuing tokens',
                                            True)

ISSUER_URL = ConfigVariableDeclaration('ISSUER_URL', None, str, 'OIDC Issuer URL', True)

LOCAL_DEFAULT_RESOURCE_PREFIX = ConfigVariableDeclaration('LOCAL_DEFAULT_RESOURCE_PREFIX', None, str,
                                                          'Prefix for building local model binary storage path',
                                                          False)

# Images registry
MODEL_IMAGES_REGISTRY_HOST = ConfigVariableDeclaration('MODEL_IMAGES_REGISTRY_HOST', None, str,
                                                       'Default registry for API server',
                                                       False)

# API server configuration
CLUSTER_CONFIG_PATH = ConfigVariableDeclaration('CLUSTER_CONFIG_PATH', None, str,
                                                'Path to cluster config file for API service',
                                                False)
CLUSTER_SECRETS_PATH = ConfigVariableDeclaration('CLUSTER_SECRETS_PATH', None, str,
                                                 'Path to cluster secrets folder for API service',
                                                 False)
JWT_CONFIG_PATH = ConfigVariableDeclaration('JWT_CONFIG_PATH', None, str,
                                            'Path to JWT config folder',
                                            False)

# API and Model invocation configuration control
FLASK_APP_SETTINGS_FILES = ConfigVariableDeclaration('FLASK_APP_SETTINGS_FILES', None, str,
                                                     'Path to flask\'s configuration file',
                                                     False)

# Model deploying configuration
MODEL_INSTANCE_SERVICE_ACCOUNT_NAME = ConfigVariableDeclaration('MODEL_INSTANCE_SERVICE_ACCOUNT_NAME', 'model', str,
                                                                'Name of K8S ServiceAccount for model instances',
                                                                False)

# Model execution control
MODEL_PROPERTIES_CACHE_TTL = ConfigVariableDeclaration('MODEL_PROPERTIES_CACHE_TTL', 10, int,
                                                       'TTL of in-memory model properties cache',
                                                       False)

# Kubernetes API
NAMESPACE = ConfigVariableDeclaration('NAMESPACE', 'odahuflow', str,
                                      'Name of kubernetes namespace inside Pod',
                                      False)

POD_NAME = ConfigVariableDeclaration('POD_NAME', None, str, 'Current pod name', False)

K8S_API_RETRY_NUMBER_MAX_LIMIT = ConfigVariableDeclaration('K8S_API_RETRY_NUMBER_MAX_LIMIT', 5, int,
                                                           'Count of retries for K8S API calls',
                                                           False)
K8S_API_RETRY_DELAY_SEC = ConfigVariableDeclaration('K8S_API_RETRY_DELAY_SEC', 3, int,
                                                    'Time wait before next retry to call K8S API',
                                                    False)

LOCAL_DEPLOY_HOSTNAME = ConfigVariableDeclaration('LOCAL_DEPLOY_HOSTNAME', 'http://localhost', str,
                                                  'Name of host on which local deployed models will be accessable',
                                                  True)

# Utils
TEMP_DIRECTORY = ConfigVariableDeclaration('TEMP_DIRECTORY', None, str,
                                           'Override system folder for temporary files',
                                           True)

# For using in tests scenarios
MODEL_NAME = ConfigVariableDeclaration('MODEL_NAME', None, str, 'Name of current model', False)

MODEL_VERSION = ConfigVariableDeclaration('MODEL_VERSION', None, str, 'Version of current model', False)

MODEL_K8S_MEMORY = ConfigVariableDeclaration('MODEL_K8S_MEMORY', '256Mi', str, 'Default k8s memory for a model', True)
MODEL_K8S_CPU = ConfigVariableDeclaration('MODEL_K8S_CPU', '256m', str, 'Default k8s cpu for a model', True)
REDUCE_MODEL_REQUESTS_BY = ConfigVariableDeclaration('REDUCE_MODEL_REQUESTS_BY', 33, int,
                                                     'Reduce k8s resource for model by specific percent', True)

# Auth
ODAHUFLOWCTL_OAUTH_CLIENT_ID = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_CLIENT_ID', 'legion-cli', str,
                                                      'Set OAuth2 Client id',
                                                      True)

ODAHUFLOWCTL_OAUTH_CLIENT_SECRET = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_CLIENT_SECRET', '', str,
                                                          'Set OAuth2 Client id',
                                                          True)

ODAHUFLOWCTL_OAUTH_SCOPE = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_SCOPE',
                                                  'openid profile email offline_access groups', str,
                                                  'Set OAuth2 scope',
                                                  True)

ODAHUFLOWCTL_OAUTH_LOOPBACK_HOST = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_LOOPBACK_HOST',
                                                          '127.0.0.1', str,
                                                          'Target redirect for OAuth2 interactive authorization',
                                                          True)

ODAHUFLOWCTL_OAUTH_LOOPBACK_URL = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_LOOPBACK_URL',
                                                         '/oauth/callback', str,
                                                         'Target redirect url for OAuth2 interactive authorization',
                                                         True)

ODAHUFLOWCTL_OAUTH_TOKEN_ISSUING_URL = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_TOKEN_ISSUING_URL',
                                                              '', str,
                                                              'OAuth2 token issuing URL',
                                                                 True)

ODAHUFLOWCTL_OAUTH_AUTH_URL = ConfigVariableDeclaration('ODAHUFLOWCTL_OAUTH_AUTH_URL',
                                                     '',
                                                     str,
                                                     'OAuth2 token issuing URL',
                                                     True)

JUPYTER_REDIRECT_URL = ConfigVariableDeclaration('JUPYTER_REDIRECT_URL',
                                                 '', str,
                                                 'JupyterLab external URL',
                                                 True)

ODAHUFLOWCTL_NONINTERACTIVE = ConfigVariableDeclaration('ODAHUFLOWCTL_NONINTERACTIVE', False,
                                                     bool, 'Disable any interaction (e.g. authorization)', True)

# Local

LOCAL_MODEL_OUTPUT_DIR = ConfigVariableDeclaration('LOCAL_MODEL_OUTPUT_DIR',
                                                   Path.home().joinpath(".odahuflow", "training_output"),
                                                   str, 'Directory where model artifacts will be saved', True)



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class FeedbackModelFeedbackResponse(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self):  # noqa: E501
        """FeedbackModelFeedbackResponse - a model defined in Swagger

        """
        self.swagger_types = {
        }

        self.attribute_map = {
        }

    @classmethod
    def from_dict(cls, dikt) -> 'FeedbackModelFeedbackResponse':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The feedback.ModelFeedbackResponse of this FeedbackModelFeedbackResponse.  # noqa: E501
        :rtype: FeedbackModelFeedbackResponse
        """
        return util.deserialize_model(dikt, cls)



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.model_packaging_result import ModelPackagingResult  # noqa: F401,E501
from odahuflow.sdk.models import util


class ModelPackagingStatus(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, created_at: str=None, exit_code: int=None, message: str=None, pod_name: str=None, reason: str=None, results: List[ModelPackagingResult]=None, state: str=None, updated_at: str=None):  # noqa: E501
        """ModelPackagingStatus - a model defined in Swagger

        :param created_at: The created_at of this ModelPackagingStatus.  # noqa: E501
        :type created_at: str
        :param exit_code: The exit_code of this ModelPackagingStatus.  # noqa: E501
        :type exit_code: int
        :param message: The message of this ModelPackagingStatus.  # noqa: E501
        :type message: str
        :param pod_name: The pod_name of this ModelPackagingStatus.  # noqa: E501
        :type pod_name: str
        :param reason: The reason of this ModelPackagingStatus.  # noqa: E501
        :type reason: str
        :param results: The results of this ModelPackagingStatus.  # noqa: E501
        :type results: List[ModelPackagingResult]
        :param state: The state of this ModelPackagingStatus.  # noqa: E501
        :type state: str
        :param updated_at: The updated_at of this ModelPackagingStatus.  # noqa: E501
        :type updated_at: str
        """
        self.swagger_types = {
            'created_at': str,
            'exit_code': int,
            'message': str,
            'pod_name': str,
            'reason': str,
            'results': List[ModelPackagingResult],
            'state': str,
            'updated_at': str
        }

        self.attribute_map = {
            'created_at': 'createdAt',
            'exit_code': 'exitCode',
            'message': 'message',
            'pod_name': 'podName',
            'reason': 'reason',
            'results': 'results',
            'state': 'state',
            'updated_at': 'updatedAt'
        }

        self._created_at = created_at
        self._exit_code = exit_code
        self._message = message
        self._pod_name = pod_name
        self._reason = reason
        self._results = results
        self._state = state
        self._updated_at = updated_at

    @classmethod
    def from_dict(cls, dikt) -> 'ModelPackagingStatus':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ModelPackagingStatus of this ModelPackagingStatus.  # noqa: E501
        :rtype: ModelPackagingStatus
        """
        return util.deserialize_model(dikt, cls)

    @property
    def created_at(self) -> str:
        """Gets the created_at of this ModelPackagingStatus.


        :return: The created_at of this ModelPackagingStatus.
        :rtype: str
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at: str):
        """Sets the created_at of this ModelPackagingStatus.


        :param created_at: The created_at of this ModelPackagingStatus.
        :type created_at: str
        """

        self._created_at = created_at

    @property
    def exit_code(self) -> int:
        """Gets the exit_code of this ModelPackagingStatus.

        Pod exit code  # noqa: E501

        :return: The exit_code of this ModelPackagingStatus.
        :rtype: int
        """
        return self._exit_code

    @exit_code.setter
    def exit_code(self, exit_code: int):
        """Sets the exit_code of this ModelPackagingStatus.

        Pod exit code  # noqa: E501

        :param exit_code: The exit_code of this ModelPackagingStatus.
        :type exit_code: int
        """

        self._exit_code = exit_code

    @property
    def message(self) -> str:
        """Gets the message of this ModelPackagingStatus.

        Pod last log  # noqa: E501

        :return: The message of this ModelPackagingStatus.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message: str):
        """Sets the message of this ModelPackagingStatus.

        Pod last log  # noqa: E501

        :param message: The message of this ModelPackagingStatus.
        :type message: str
        """

        self._message = message

    @property
    def pod_name(self) -> str:
        """Gets the pod_name of this ModelPackagingStatus.

        Pod package for name  # noqa: E501

        :return: The pod_name of this ModelPackagingStatus.
        :rtype: str
        """
        return self._pod_name

    @pod_name.setter
    def pod_name(self, pod_name: str):
        """Sets the pod_name of this ModelPackagingStatus.

        Pod package for name  # noqa: E501

        :param pod_name: The pod_name of this ModelPackagingStatus.
        :type pod_name: str
        """

        self._pod_name = pod_name

    @property
    def reason(self) -> str:
        """Gets the reason of this ModelPackagingStatus.

        Pod reason  # noqa: E501

        :return: The reason of this ModelPackagingStatus.
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason: str):
        """Sets the reason of this ModelPackagingStatus.

        Pod reason  # noqa: E501

        :param reason: The reason of this ModelPackagingStatus.
        :type reason: str
        """

        self._reason = reason

    @property
    def results(self) -> List[ModelPackagingResult]:
        """Gets the results of this ModelPackagingStatus.

        List of packaing results  # noqa: E501

        :return: The results of this ModelPackagingStatus.
        :rtype: List[ModelPackagingResult]
        """
        return self._results

    @results.setter
    def results(self, results: List[ModelPackagingResult]):
        """Sets the results of this ModelPackagingStatus.

        List of packaing results  # noqa: E501

        :param results: The results of this ModelPackagingStatus.
        :type results: List[ModelPackagingResult]
        """

        self._results = results

    @property
    def state(self) -> str:
        """Gets the state of this ModelPackagingStatus.

        Model Packaging State  # noqa: E501

        :return: The state of this ModelPackagingStatus.
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state: str):
        """Sets the state of this ModelPackagingStatus.

        Model Packaging State  # noqa: E501

        :param state: The state of this ModelPackagingStatus.
        :type state: str
        """

        self._state = state

    @property
    def updated_at(self) -> str:
        """Gets the updated_at of this ModelPackagingStatus.


        :return: The updated_at of this ModelPackagingStatus.
        :rtype: str
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at: str):
        """Sets the updated_at of this ModelPackagingStatus.


        :param updated_at: The updated_at of this ModelPackagingStatus.
        :type updated_at: str
        """

        self._updated_at = updated_at



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.common_configuration import CommonConfiguration  # noqa: F401,E501
from odahuflow.sdk.models.training_configuration import TrainingConfiguration  # noqa: F401,E501
from odahuflow.sdk.models import util


class Configuration(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, common: CommonConfiguration=None, training: TrainingConfiguration=None):  # noqa: E501
        """Configuration - a model defined in Swagger

        :param common: The common of this Configuration.  # noqa: E501
        :type common: CommonConfiguration
        :param training: The training of this Configuration.  # noqa: E501
        :type training: TrainingConfiguration
        """
        self.swagger_types = {
            'common': CommonConfiguration,
            'training': TrainingConfiguration
        }

        self.attribute_map = {
            'common': 'common',
            'training': 'training'
        }

        self._common = common
        self._training = training

    @classmethod
    def from_dict(cls, dikt) -> 'Configuration':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Configuration of this Configuration.  # noqa: E501
        :rtype: Configuration
        """
        return util.deserialize_model(dikt, cls)

    @property
    def common(self) -> CommonConfiguration:
        """Gets the common of this Configuration.

        Common secretion of configuration  # noqa: E501

        :return: The common of this Configuration.
        :rtype: CommonConfiguration
        """
        return self._common

    @common.setter
    def common(self, common: CommonConfiguration):
        """Sets the common of this Configuration.

        Common secretion of configuration  # noqa: E501

        :param common: The common of this Configuration.
        :type common: CommonConfiguration
        """

        self._common = common

    @property
    def training(self) -> TrainingConfiguration:
        """Gets the training of this Configuration.

        Configuration describe training process  # noqa: E501

        :return: The training of this Configuration.
        :rtype: TrainingConfiguration
        """
        return self._training

    @training.setter
    def training(self, training: TrainingConfiguration):
        """Sets the training of this Configuration.

        Configuration describe training process  # noqa: E501

        :param training: The training of this Configuration.
        :type training: TrainingConfiguration
        """

        self._training = training



#
#    Copyright 2019 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
Aggregated API client (can apply multiple resources)
"""
import asyncio
import json
import logging
import os
import typing
from os import listdir
from os.path import isdir, join, isfile

import yaml
from odahuflow.sdk.clients.api import RemoteAPIClient, WrongHttpStatusCode, AsyncRemoteAPIClient
from odahuflow.sdk.clients.connection import ConnectionClient, AsyncConnectionClient
from odahuflow.sdk.clients.deployment import ModelDeploymentClient, ModelDeployment, AsyncModelDeploymentClient
from odahuflow.sdk.clients.packaging import ModelPackagingClient, AsyncModelPackagingClient
from odahuflow.sdk.clients.packaging_integration import PackagingIntegrationClient, AsyncPackagingIntegrationClient
from odahuflow.sdk.clients.route import ModelRoute, ModelRouteClient, AsyncModelRouteClient
from odahuflow.sdk.clients.toolchain_integration import ToolchainIntegrationClient, AsyncToolchainIntegrationClient
from odahuflow.sdk.clients.training import ModelTrainingClient, ModelTraining, AsyncModelTrainingClient
from odahuflow.sdk.models import Connection, ToolchainIntegration, ModelPackaging, PackagingIntegration

TARGET_CLASSES = {
    'ModelTraining': ModelTraining,
    'ToolchainIntegration': ToolchainIntegration,
    'ModelDeployment': ModelDeployment,
    'ModelRoute': ModelRoute,
    'Connection': Connection,
    'ModelPackaging': ModelPackaging,
    'PackagingIntegration': PackagingIntegration,
}

LOGGER = logging.getLogger(__name__)


class InvalidResourceType(Exception):
    """
    Invalid resource type (unsupported) exception
    """

    pass


class OdahuflowCloudResourceUpdatePair(typing.NamedTuple):
    """
    Information about resources to update
    """

    resource_id: str
    resource: object


class OdahuflowCloudResourcesUpdateList(typing.NamedTuple):
    """
    Bulk update request (multiple resources)
    """

    changes: typing.Tuple[OdahuflowCloudResourceUpdatePair] = tuple()


class ApplyResult(typing.NamedTuple):
    """
    Result of bulk applying
    """

    created: typing.Tuple[OdahuflowCloudResourceUpdatePair] = tuple()
    removed: typing.Tuple[OdahuflowCloudResourceUpdatePair] = tuple()
    changed: typing.Tuple[OdahuflowCloudResourceUpdatePair] = tuple()
    errors: typing.Tuple[Exception] = tuple()


# pylint: disable=R0911
def build_client(resource: OdahuflowCloudResourceUpdatePair, api_client: RemoteAPIClient) -> typing.Optional[object]:
    """
    Build client for particular resource (e.g. it builds ModelTrainingClient for ModelTraining resource)

    :param resource: target resource
    :param api_client: base API client to extract connection options from
    :return: remote client or None
    """
    if isinstance(resource.resource, ModelTraining):
        return ModelTrainingClient.construct_from_other(api_client)
    elif isinstance(resource.resource, ModelDeployment):
        return ModelDeploymentClient.construct_from_other(api_client)
    elif isinstance(resource.resource, Connection):
        return ConnectionClient.construct_from_other(api_client)
    elif isinstance(resource.resource, ToolchainIntegration):
        return ToolchainIntegrationClient.construct_from_other(api_client)
    elif isinstance(resource.resource, ModelRoute):
        return ModelRouteClient.construct_from_other(api_client)
    elif isinstance(resource.resource, ModelPackaging):
        return ModelPackagingClient.construct_from_other(api_client)
    elif isinstance(resource.resource, PackagingIntegration):
        return PackagingIntegrationClient.construct_from_other(api_client)
    else:
        raise InvalidResourceType('{!r} is invalid resource '.format(resource.resource))


# pylint: disable=R0911
def build_async_client(resource: OdahuflowCloudResourceUpdatePair,
                       async_api_client: AsyncRemoteAPIClient
                       ) -> typing.Optional[object]:
    """
    Build client for particular resource (e.g. it builds ModelTrainingClient for ModelTraining resource)

    :param resource: target resource
    :param async_api_client: base async API client to extract connection options from
    :return: remote client or None
    """
    if isinstance(resource.resource, ModelTraining):
        return AsyncModelTrainingClient.construct_from_other(async_api_client)
    elif isinstance(resource.resource, ModelDeployment):
        return AsyncModelDeploymentClient.construct_from_other(async_api_client)
    elif isinstance(resource.resource, Connection):
        return AsyncConnectionClient.construct_from_other(async_api_client)
    elif isinstance(resource.resource, ToolchainIntegration):
        return AsyncToolchainIntegrationClient.construct_from_other(async_api_client)
    elif isinstance(resource.resource, ModelRoute):
        return AsyncModelRouteClient.construct_from_other(async_api_client)
    elif isinstance(resource.resource, ModelPackaging):
        return AsyncModelPackagingClient.construct_from_other(async_api_client)
    elif isinstance(resource.resource, PackagingIntegration):
        return AsyncPackagingIntegrationClient.construct_from_other(async_api_client)
    else:
        raise InvalidResourceType('{!r} is invalid resource '.format(resource.resource))


def build_resource(declaration: dict) -> OdahuflowCloudResourceUpdatePair:
    """
    Build resource from it's declaration

    :param declaration: declaration of resource
    :return: built resource
    """
    resource_type = declaration.get('kind')
    if resource_type is None:
        raise Exception(f'Kind of object {declaration} must be not null')

    if not isinstance(resource_type, str):
        raise Exception(f'Kind of object {declaration} should be string')

    if resource_type not in TARGET_CLASSES:
        raise Exception(f'Unknown kind of object: \'{resource_type}\'')

    resource = TARGET_CLASSES[resource_type].from_dict(declaration)

    return OdahuflowCloudResourceUpdatePair(
        resource_id=resource.id,
        resource=resource
    )


def parse_stream(data_stream: typing.TextIO) -> OdahuflowCloudResourcesUpdateList:
    """
    Parse YAML/JSON TextIO for Odahuflow resources

    :param data_stream: text stream with yaml/json data
    :raises Exception: if parsing of file is impossible
    :raises ValueError: if invalid Odahuflow resource detected
    :return: parsed resources
    """
    try:
        # Technically YAML is a superset of JSON.
        # So we have to try parse as json first.
        items = json.load(data_stream)

        LOGGER.debug('Successfully parsed as JSON format')
    except json.JSONDecodeError:
        try:
            # Return to the beginning of the stream
            data_stream.seek(0)

            items = tuple(yaml.load_all(data_stream, Loader=yaml.SafeLoader))

            LOGGER.debug('Successfully parsed as YAML format')
        except yaml.YAMLError:
            raise Exception('not valid JSON or YAML')

    if not isinstance(items, (list, tuple)):
        items = [items]

    result: typing.List[OdahuflowCloudResourceUpdatePair] = []

    for item in items:
        if item is None:
            continue

        if not isinstance(item, dict):
            raise ValueError(f'Invalid Odahuflow resource in file: {item}')

        result.append(build_resource(item))

    return OdahuflowCloudResourcesUpdateList(
        changes=tuple(result)
    )


def parse_resources_dir(path: str) -> typing.List[OdahuflowCloudResourceUpdatePair]:
    """
    Parse dir with YAML/JSON files for Odahuflow resources

    :param path: path to file (local)
    :raises Exception: if parsing of file is impossible
    :raises ValueError: if invalid Odahuflow resource detected
    :return: parsed resources
    """
    if not os.path.exists(path):
        raise FileNotFoundError(f'Resource directory {path} not found')

    if not isdir(path):
        raise FileNotFoundError(f'{path} is not a directory')

    resource_files = [join(path, f) for f in listdir(path) if
                      isfile(join(path, f))]

    entities: typing.List[OdahuflowCloudResourceUpdatePair] = []
    for file in resource_files:
        try:
            LOGGER.debug(f'Parsing the {file} file')
            entities.extend(parse_resources_file(file).changes)
        except Exception:
            LOGGER.exception('parse error')

    return entities


def parse_resources_file(path: str) -> OdahuflowCloudResourcesUpdateList:
    """
    Parse file (YAML/JSON) for Odahuflow resources

    :param path: path to file (local)
    :raises Exception: if parsing of file is impossible
    :raises ValueError: if invalid Odahuflow resource detected
    :return: parsed resources
    """
    if not os.path.exists(path):
        raise FileNotFoundError(f'Resource file \'{path}\' not found')

    with open(path, 'r') as data_stream:
        return parse_stream(data_stream)


def parse_resources_file_with_one_item(path: str) -> OdahuflowCloudResourceUpdatePair:
    """
    Parse file (YAML/JSON) for Odahuflow resource. Raise exception if it is more then one resource

    :param path: path to file (local)
    :raises Exception: if parsing of file is impossible
    :raises Exception: if more then one resource found
    :raises ValueError: if invalid Odahuflow resource detected
    :return: parsed resource
    """
    resources = parse_resources_file(path)
    if len(resources.changes) != 1:
        raise Exception('{!r} should contain 1 item, but {!r} founded'.format(path, len(resources)))
    return resources.changes[0]


async def async_apply(updates: OdahuflowCloudResourcesUpdateList,
                      async_api_client: AsyncRemoteAPIClient,
                      is_removal: bool) -> ApplyResult:
    """
    Apply changes on Odahuflow cloud

    :param updates: changes to apply
    :param async_api_client: client to extract connection properties from
    :param is_removal: is it removal?
    :return: result of applying
    """
    created = []
    removed = []
    changed = []
    errors = []

    # Operate over all resources
    for idx, change in enumerate(updates.changes):
        resource_str_identifier = f'#{idx + 1}. {change.resource_id}' if change.resource_id else f'#{idx + 1}'

        LOGGER.debug('Processing resource %r', resource_str_identifier)
        # Build and check client
        try:
            client = build_async_client(change, async_api_client)
        except Exception as general_exception:
            errors.append(Exception(f'Can not get build client for {resource_str_identifier}: {general_exception}'))
            continue

        # Check is resource exist or not
        try:
            await client.get(change.resource_id)
            resource_exist = True
        except WrongHttpStatusCode as http_exception:
            if http_exception.status_code == 404:
                resource_exist = False
            else:
                errors.append(Exception(f'Can not get status of resource '
                                        f'{resource_str_identifier}: {http_exception}'))
                continue
        except Exception as general_exception:
            errors.append(Exception(f'Can not get status of resource '
                                    f'{resource_str_identifier}: {general_exception}'))
            continue

        # Change resource (update/create/delete)
        try:
            # If not removal (creation / update)
            if not is_removal:
                if resource_exist:
                    LOGGER.info('Editing of #%d %s (name: %s)', idx + 1, change.resource, change.resource_id)
                    await client.edit(change.resource)
                    changed.append(change)
                else:
                    LOGGER.info('Creating of #%d %s (name: %s)', idx + 1, change.resource, change.resource_id)
                    await client.create(change.resource)
                    created.append(change)
            # If removal
            else:
                # Only if resource exists on a cluster
                if resource_exist:
                    LOGGER.info('Removing of #%d %s (name: %s)', idx + 1, change.resource, change.resource_id)
                    await client.delete(change.resource_id)
                    removed.append(change)
        except Exception as general_exception:
            errors.append(Exception(f'Can not update resource {resource_str_identifier}: {general_exception}'))
            continue

    return ApplyResult(tuple(created), tuple(removed), tuple(changed), tuple(errors))


def apply(updates: OdahuflowCloudResourcesUpdateList,
          api_client: typing.Union[AsyncRemoteAPIClient, RemoteAPIClient],
          is_removal: bool) -> ApplyResult:
    """
    Apply changes on Odahuflow cloud (wrapper for async_apply). Used for not async client (For backward compatibility)

    :param updates: changes to apply
    :param api_client: client to extract connection properties from
    :param is_removal: is it removal?
    :return:  result of applying
    """
    loop = asyncio.get_event_loop()

    feature = async_apply(updates, api_client, is_removal)

    result = loop.run_until_complete(feature)

    return result



#
#    Copyright 2017 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
API client
"""
import typing

from odahuflow.sdk.clients.api import RemoteAPIClient, AsyncRemoteAPIClient
from odahuflow.sdk.definitions import PACKING_INTEGRATION_URL
from odahuflow.sdk.models import PackagingIntegration


class PackagingIntegrationClient(RemoteAPIClient):
    """
    HTTP packaging integration client
    """

    def get(self, name: str) -> PackagingIntegration:
        """
        Get Packaging Integration from API server

        :param name: Packaging Integration name
        :type name: str
        :return: Packaging Integration
        """
        return PackagingIntegration.from_dict(self.query(f'{PACKING_INTEGRATION_URL}/{name}'))

    def get_all(self) -> typing.List[PackagingIntegration]:
        """
        Get all Packaging Integrations from API server

        :return: all Packaging Integrations
        """
        return [PackagingIntegration.from_dict(mr) for mr in self.query(PACKING_INTEGRATION_URL)]

    def create(self, mr: PackagingIntegration) -> PackagingIntegration:
        """
        Create Packaging Integration

        :param mr: Packaging Integration
        :return Message from API server
        """
        return PackagingIntegration.from_dict(
            self.query(PACKING_INTEGRATION_URL, action='POST', payload=mr.to_dict())
        )

    def edit(self, mr: PackagingIntegration) -> PackagingIntegration:
        """
        Edit Packaging Integration

        :param mr: Packaging Integration
        :return Message from API server
        """
        return PackagingIntegration.from_dict(
            self.query(PACKING_INTEGRATION_URL, action='PUT', payload=mr.to_dict())
        )

    def delete(self, name: str) -> str:
        """
        Delete Packaging Integrations

        :param name: Name of a Packaging Integration
        :return Message from API server
        """
        return self.query(f'{PACKING_INTEGRATION_URL}/{name}', action='DELETE')


class AsyncPackagingIntegrationClient(AsyncRemoteAPIClient):
    """
    HTTP packaging integration async client
    """

    async def get(self, name: str) -> PackagingIntegration:
        """
        Get Packaging Integration from API server

        :param name: Packaging Integration name
        :type name: str
        :return: Packaging Integration
        """
        return PackagingIntegration.from_dict(await self.query(f'{PACKING_INTEGRATION_URL}/{name}'))

    async def get_all(self) -> typing.List[PackagingIntegration]:
        """
        Get all Packaging Integrations from API server

        :return: all Packaging Integrations
        """
        return [PackagingIntegration.from_dict(mr) for mr in await self.query(PACKING_INTEGRATION_URL)]

    async def create(self, mr: PackagingIntegration) -> PackagingIntegration:
        """
        Create Packaging Integration

        :param mr: Packaging Integration
        :return Message from API server
        """
        return PackagingIntegration.from_dict(
            await self.query(PACKING_INTEGRATION_URL, action='POST', payload=mr.to_dict())
        )

    async def edit(self, mr: PackagingIntegration) -> PackagingIntegration:
        """
        Edit Packaging Integration

        :param mr: Packaging Integration
        :return Message from API server
        """
        return PackagingIntegration.from_dict(
            await self.query(PACKING_INTEGRATION_URL, action='PUT', payload=mr.to_dict())
        )

    async def delete(self, name: str) -> str:
        """
        Delete Packaging Integrations

        :param name: Name of a Packaging Integration
        :return Message from API server
        """
        return await self.query(f'{PACKING_INTEGRATION_URL}/{name}', action='DELETE')



#
#    Copyright 2019 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
Logger setup
"""
import logging
import sys

from odahuflow.sdk import config

ENABLED_VERBOSE_LOG_LEVEL = logging.DEBUG
DISABLED_VERBOSE_LOG_LEVEL = logging.CRITICAL


def configure_logging(verbose: bool):
    """
    Set appropriate log level

    :param verbose: logging level
    """
    if verbose or config.DEBUG:
        log_level = ENABLED_VERBOSE_LOG_LEVEL
    else:
        log_level = DISABLED_VERBOSE_LOG_LEVEL

    logging.basicConfig(level=log_level,
                        format='%(asctime)s - %(levelname)s - %(message)s',
                        stream=sys.stderr)


def is_verbose_enabled() -> bool:
    """
    Determine verbose flag by logging level
    @return: true if verbose flag is provided
    """
    return logging.root.level == ENABLED_VERBOSE_LOG_LEVEL



#
#    Copyright 2019 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
OAuth2 handler
"""
import base64
import socket
import logging
from http.server import BaseHTTPRequestHandler, HTTPServer
from threading import Thread
from contextlib import closing
from urllib.parse import parse_qs, urlparse
import typing
import json

import requests

from odahuflow.sdk import config
from odahuflow.sdk.utils import render_template

LOGGER = logging.getLogger()

SA_SCOPE = 'openid profile offline_access groups'


class OAuthLoginResult(typing.NamedTuple):
    """
    Result of oauth login process
    """

    access_token: str
    refresh_token: str
    id_token: str
    issuing_url: str
    user_email: str
    user_name: str


def find_free_port(bind_addr: str = '0.0.0.0') -> int:
    """
    Find next available port on local machine

    :return: int - port number
    """
    # pylint: disable=E1101
    # due to bug with closing return type
    LOGGER.debug('Trying to get free port')
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        sock.bind((bind_addr, 0))
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        port = sock.getsockname()[1]
        LOGGER.debug('Free port %d has been found', port)
        return port


def _try_to_extract_issuing_url_from_well_known_metadata(well_known_address: str) -> typing.Optional[str]:
    """
    Try to extract token issuing url from well-known location

    :param well_known_address: well-known URL
    :type well_known_address: str
    :return: str or None -- token issuing URL
    """
    try:
        LOGGER.debug('Trying to extract well-known information from address %r', well_known_address)
        response = requests.get(url=well_known_address)
        data = response.json()
    except requests.HTTPError as http_error:
        LOGGER.debug('Failed to extract well-known information from address %r - %s', well_known_address, http_error)
        return None
    except ValueError as value_error:
        LOGGER.debug('Failed to parse well-known information from address %r - %s', well_known_address, value_error)
        return None

    token_endpoint = data.get('token_endpoint')
    if not token_endpoint:
        LOGGER.debug('well-known information does not contain token_endpoint (%s)', well_known_address)
        return

    return token_endpoint


def get_oauth_token_issuer_url(redirect_url: str) -> typing.Optional[str]:
    """
    Get OAuth2 token issuing URL

    :param redirect_url: current redirection URL
    :type redirect_url: str
    :return: str or None -- token issuing URL
    """
    # 1st priority - check config variable ODAHUFLOWCTL_OAUTH_TOKEN_ISSUING_URL
    if config.ODAHUFLOWCTL_OAUTH_TOKEN_ISSUING_URL:
        return config.ODAHUFLOWCTL_OAUTH_TOKEN_ISSUING_URL

    # 2nd priority - try to remove URL parts of redirect URL and append /.well-known/openid-configuration
    # According to https://tools.ietf.org/pdf/rfc8414.pdf
    loc = urlparse(redirect_url)
    path_parts = loc.path.strip('/').split('/')

    for i in range(len(path_parts)):
        sub_path = '/'.join(path_parts[0:i])
        full_uri = f'{loc.scheme}://{loc.netloc}/{sub_path}/.well-known/openid-configuration'
        endpoint = _try_to_extract_issuing_url_from_well_known_metadata(full_uri)
        if endpoint:
            return endpoint
    return None


def _ask_token_endpoint(url: str, payload: typing.Any) -> typing.Optional[OAuthLoginResult]:
    """
    Query token endpoint to refresh / issue new token

    :param url: token endpoint
    :param payload: query payload
    :return: OAuthLoginResult or None -- login result or None
    """
    try:
        res = requests.post(url, data=payload)
        data = res.json()
    except requests.HTTPError as http_error:
        LOGGER.warning('Failed to get ID token on %r - %s', url, http_error)
        return None
    except ValueError as value_error:
        LOGGER.warning('Failed to get ID token on %r - %s', url, value_error)
        return None

    access_token, refresh_token, id_token = data.get('access_token'), data.get('refresh_token'), data.get('id_token')
    if not access_token or not refresh_token or not id_token:
        LOGGER.warning('Response does not contain access_token / refresh_token / id_token')
        return None

    _, body, _ = id_token.split('.')
    id_payload = json.loads(base64.b64decode(body + '===').decode('utf-8'))
    user_name = id_payload.get('name')
    user_email = id_payload.get('email')

    result = OAuthLoginResult(
        access_token=access_token,
        refresh_token=refresh_token,
        id_token=id_token,
        issuing_url=url,
        user_name=user_name,
        user_email=user_email
    )

    LOGGER.debug('Token information for %s / %s has been received', result.user_name, result.user_email)
    return result


def do_refresh_token(refresh_token: str, issue_token_url: str) -> typing.Optional[OAuthLoginResult]:
    """
    Refresh token using previously saved refresh_token

    :param refresh_token: refresh token
    :param issue_token_url: issue token URL
    :return: OAuthLoginResult or None -- refresh result or None
    """
    LOGGER.debug('Trying to refresh ID token using %s', issue_token_url)

    payload = {
        'grant_type': 'refresh_token',
        'client_id': config.ODAHUFLOWCTL_OAUTH_CLIENT_ID,
        'client_secret': config.ODAHUFLOWCTL_OAUTH_CLIENT_SECRET,
        'refresh_token': refresh_token
    }
    return _ask_token_endpoint(issue_token_url, payload)


def do_client_cred_authentication(
        client_id: str = config.ODAHUFLOWCTL_OAUTH_CLIENT_ID,
        client_secret: str = config.ODAHUFLOWCTL_OAUTH_CLIENT_SECRET,
        issue_token_url: str = config.API_ISSUING_URL
):
    """
    Get access and id token using oauth2 client credentials fow
    :param client_id:
    :param client_secret:
    :param issue_token_url:
    :return:
    """
    LOGGER.debug('Trying to get ID token via Client Credentials Flow using %s', issue_token_url)

    payload = {
        'grant_type': 'client_credentials',
        'client_id': client_id,
        'client_secret': client_secret,
        'scope': SA_SCOPE
    }
    return _ask_token_endpoint(issue_token_url, payload)


def get_id_token(code: str, issue_token_url: str, redirect_uri: str) -> typing.Optional[OAuthLoginResult]:
    """
    Get ID token and validate received data

    :param code: code
    :param issue_token_url: issuing URL
    :param redirect_uri: redirect URL
    :return: OAuthLoginResult or None -- login result or None
    """
    LOGGER.debug('Trying to get ID token and validate using %s', issue_token_url)

    payload = {
        'grant_type': 'authorization_code',
        'client_id': config.ODAHUFLOWCTL_OAUTH_CLIENT_ID,
        'code': code,
        'redirect_uri': redirect_uri
    }

    if config.ODAHUFLOWCTL_OAUTH_CLIENT_SECRET:
        payload['client_secret'] = config.ODAHUFLOWCTL_OAUTH_CLIENT_SECRET

    return _ask_token_endpoint(issue_token_url, payload)


class OAuth2Handler(BaseHTTPRequestHandler):
    """
    Handler for simple loopback listening server that handles OAuth2 redirects to loopback host
    """

    def __init__(self, *args, on_token_received=None, state=None, target_url=None, redirect_url=None, **kwargs):
        """
        Initialize loopback server

        :param args: system args
        :param on_token_received: callback that should be called on final auth. stage (when all tokens are received)
        :param state: randomly generated token for OAuth2 pipeline
        :param target_url: captured redirect to IP's URL
        :param redirect_url:  redirect URL to continue authorization
        :param kwargs: system args
        """
        self.on_token_received = on_token_received
        self.state = state
        self.target_url = target_url
        self.redirect_url = redirect_url
        BaseHTTPRequestHandler.__init__(self, *args)

    def log_message(self, format: str, *args: typing.Tuple[typing.Any, ...]) -> None:  # pylint: disable=W0622
        """
        Log an arbitrary message.

        The first argument, FORMAT, is a format string for the
        message to be logged.  If the format string contains
        any % escapes requiring parameters, they should be
        specified as subsequent arguments (it's just like
        printf!).

        :param format: format
        :param args: arguments for format
        :return: None
        """
        LOGGER.debug('%s - %s', self.address_string(), format % args)

    def raise_error(self, message: str) -> None:
        """
        Raise error if it is a problem

        :param message: error description
        :type message: str
        :return: None
        """
        self.send_response(500)
        self.send_header('Content-type', 'text/plain')
        self.end_headers()
        self.wfile.write(message.encode('utf-8'))

    def do_GET(self) -> None:
        """
        Handle GET action

        :return: None
        """
        loc = urlparse(self.path)
        if loc.path == config.ODAHUFLOWCTL_OAUTH_LOOPBACK_URL:
            params = parse_qs(loc.query)

            if 'state' not in params or len(params['state']) != 1:
                return self.raise_error('state is missed')

            if 'code' not in params or len(params['code']) != 1:
                return self.raise_error('code is missed')

            state = params['state'][0]
            code = params['code'][0]

            if state != self.state:
                return self.raise_error(f'Wrong state. Received {state!r}, expected {self.state!r}')

            issue_token_url = get_oauth_token_issuer_url(self.target_url)
            if not issue_token_url:
                return self.raise_error(f'Can not get URL for issuing long-life token from {self.target_url}')

            login_result = get_id_token(code, issue_token_url, self.redirect_url)
            if not login_result:
                return self.raise_error(f'Failed to get long-life token from {issue_token_url}')

            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.end_headers()

            content = render_template('callback-response.html', {})
            self.wfile.write(content.encode('utf-8'))

            self.on_token_received(login_result)
        else:
            self.send_response(404)
            self.end_headers()


def handler_builder(on_token_received: typing.Callable[[OAuthLoginResult], None],
                    state: str, target_url: str, redirect_url: str) -> typing.Callable:
    """
    Create handler builder for OAuth2 callback built-in server

    :param on_token_received: callback that should be called on final auth. stage (when all tokens are received)
    :param state: randomly generated token for OAuth2 pipeline
    :param target_url: captured redirect to IP's URL
    :param redirect_url:  redirect URL to continue authorization
    :return: callable - handler builder function
    """
    def init(*args, **kwargs) -> object:
        """
        Builder (builds OAuth2Handler instance)

        :param args: system args
        :param kwargs: system args
        :return: object -- handler
        """
        OAuth2Handler(*args,
                      on_token_received=on_token_received,
                      state=state,
                      target_url=target_url,
                      redirect_url=redirect_url,
                      **kwargs)
    return init


def start_oauth2_callback_handler(on_token_received: typing.Callable[[OAuthLoginResult], None],
                                  state: str, target_url: str) -> str:
    """
    Start OAuth2 callback handler

    :param on_token_received: callback that should be called on final auth. stage (when all tokens are received)
    :param state: randomly generated token for OAuth2 pipeline
    :param target_url: captured redirect to IP's URL
    :return: str -- redirect URL to continue authorization
    """
    host = config.ODAHUFLOWCTL_OAUTH_LOOPBACK_HOST
    port = find_free_port(host)
    redirect_url = f'http://{config.ODAHUFLOWCTL_OAUTH_LOOPBACK_HOST}:{port}{config.ODAHUFLOWCTL_OAUTH_LOOPBACK_URL}'

    server = HTTPServer((host, port),
                        handler_builder(on_token_received, state, target_url, redirect_url))

    callback_handler = Thread(name='oauth2_callback_handler',
                              target=server.serve_forever)
    callback_handler.start()
    return redirect_url



#
#    Copyright 2020 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
OIDC configuration tool
"""

import requests

WELL_KNOWN_CONFIGURATION_URL = '.well-known/openid-configuration'

# Some OpenID Provider configuration keys
# (defined at https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata)
ISSUER = 'issuer'
AUTHORIZATION_ENDPOINT = 'authorization_endpoint'
TOKEN_ENDPOINT = 'token_endpoint'


class OpenIdProviderConfiguration:
    """
    OpenID Provider Configuration according to https://openid.net/specs/openid-connect-discovery-1_0.html
    """

    def __init__(self, issuer: str):
        """

        :param issuer: URL to OIDC issuer
        """
        self._issuer = issuer
        self._config_json = None

    def extract_value(self, key):
        if self._config_json is None:
            raise RuntimeError('You must fetch OpenID Provider configuration via .fetch_configuration() '
                               'or .async_fetch_configuration() method before refer some properties')

        return self._config_json.get(key)

    def fetch_configuration(self) -> None:
        """
        Fetch OpenID Provider configuration making HTTP request
        :return:
        """
        response = requests.get(self.configuration_url)
        if response.status_code == 200:
            self._config_json = response.json()
        elif 400 <= response.status_code < 600:
            raise RuntimeError(f'Some error during attempt to fetch OpenID Provider configuration. '
                               f'Reason: {response.reason}')
        else:
            raise RuntimeError(f'Not expected status code: {response.status_code}')

    async def async_fetch_configuration(self) -> None:
        """
        Fetch OpenID Provider configuration making HTTP request
        :return:
        """
        raise NotImplementedError

    @property
    def configuration_url(self) -> str:
        return f'{self._issuer}/{WELL_KNOWN_CONFIGURATION_URL}'

    @property
    def issuer(self) -> str:
        return self.extract_value(ISSUER)

    @property
    def authorization_endpoint(self) -> str:
        return self.extract_value(AUTHORIZATION_ENDPOINT)

    @property
    def token_endpoint(self) -> str:
        return self.extract_value(TOKEN_ENDPOINT)



#
#    Copyright 2017 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
API client
"""
import logging
import typing
from urllib import parse

from odahuflow.sdk.clients.api import RemoteAPIClient, AsyncRemoteAPIClient
from odahuflow.sdk.definitions import MODEL_DEPLOYMENT_URL
from odahuflow.sdk.models import ModelDeployment

LOGGER = logging.getLogger(__name__)

READY_STATE = "Ready"
PROCESSING_STATE = "Processing"
FAILED_STATE = "Failed"


class ModelDeploymentClient(RemoteAPIClient):
    """
    HTTP model deployment client
    """

    def get(self, name: str) -> ModelDeployment:
        """
        Get Model Deployment from API server

        :param name: Model Deployment name
        :return: Model Deployment
        """
        return ModelDeployment.from_dict(self.query(f'{MODEL_DEPLOYMENT_URL}/{name}'))

    def get_all(self, labels: typing.Dict[str, str] = None) -> typing.List[ModelDeployment]:
        """
        Get all Model Deployments from API server

        :return: all Model Deployments
        """
        if labels:
            url = f'{MODEL_DEPLOYMENT_URL}?{parse.urlencode(labels)}'
        else:
            url = MODEL_DEPLOYMENT_URL

        return [ModelDeployment.from_dict(md) for md in self.query(url)]

    def create(self, md: ModelDeployment) -> ModelDeployment:
        """
        Create Model Deployment

        :param md: Model Deployment
        :return Message from API server
        """
        return ModelDeployment.from_dict(
            self.query(MODEL_DEPLOYMENT_URL, action='POST', payload=md.to_dict())
        )

    def edit(self, md: ModelDeployment) -> ModelDeployment:
        """
        Edit Model Deployment

        :param md: Model Deployment
        :return Message from API server
        """
        return ModelDeployment.from_dict(
            self.query(MODEL_DEPLOYMENT_URL, action='PUT', payload=md.to_dict())
        )

    def delete(self, name: str) -> str:
        """
        Delete Model Deployments

        :param name: Name of a Model Deployment
        :return Message from API server
        """
        return self.query(f'{MODEL_DEPLOYMENT_URL}/{name}', action='DELETE')['message']


class AsyncModelDeploymentClient(AsyncRemoteAPIClient):
    """
    HTTP model deployment async client
    """

    async def get(self, name: str) -> ModelDeployment:
        """
        Get Model Deployment from API server

        :param name: Model Deployment name
        :return: Model Deployment
        """
        return ModelDeployment.from_dict(await self.query(f'{MODEL_DEPLOYMENT_URL}/{name}'))

    async def get_all(self, labels: typing.Dict[str, str] = None) -> typing.List[ModelDeployment]:
        """
        Get all Model Deployments from API server

        :return: all Model Deployments
        """
        if labels:
            url = f'{MODEL_DEPLOYMENT_URL}?{parse.urlencode(labels)}'
        else:
            url = MODEL_DEPLOYMENT_URL

        return [ModelDeployment.from_dict(md) for md in await self.query(url)]

    async def create(self, md: ModelDeployment) -> ModelDeployment:
        """
        Create Model Deployment

        :param md: Model Deployment
        :return Message from API server
        """
        return ModelDeployment.from_dict(
            await self.query(MODEL_DEPLOYMENT_URL, action='POST', payload=md.to_dict())
        )

    async def edit(self, md: ModelDeployment) -> ModelDeployment:
        """
        Edit Model Deployment

        :param md: Model Deployment
        :return Message from API server
        """
        return ModelDeployment.from_dict(
            await self.query(MODEL_DEPLOYMENT_URL, action='PUT', payload=md.to_dict())
        )

    async def delete(self, name: str) -> str:
        """
        Delete Model Deployments

        :param name: Name of a Model Deployment
        :return Message from API server
        """
        return (await self.query(f'{MODEL_DEPLOYMENT_URL}/{name}', action='DELETE'))['message']



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class ExternalUrl(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, image_url: str=None, name: str=None, url: str=None):  # noqa: E501
        """ExternalUrl - a model defined in Swagger

        :param image_url: The image_url of this ExternalUrl.  # noqa: E501
        :type image_url: str
        :param name: The name of this ExternalUrl.  # noqa: E501
        :type name: str
        :param url: The url of this ExternalUrl.  # noqa: E501
        :type url: str
        """
        self.swagger_types = {
            'image_url': str,
            'name': str,
            'url': str
        }

        self.attribute_map = {
            'image_url': 'imageUrl',
            'name': 'name',
            'url': 'url'
        }

        self._image_url = image_url
        self._name = name
        self._url = url

    @classmethod
    def from_dict(cls, dikt) -> 'ExternalUrl':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ExternalUrl of this ExternalUrl.  # noqa: E501
        :rtype: ExternalUrl
        """
        return util.deserialize_model(dikt, cls)

    @property
    def image_url(self) -> str:
        """Gets the image_url of this ExternalUrl.

        Optional link to an image which represents a type of the resource, for example the logo of Grafana  # noqa: E501

        :return: The image_url of this ExternalUrl.
        :rtype: str
        """
        return self._image_url

    @image_url.setter
    def image_url(self, image_url: str):
        """Sets the image_url of this ExternalUrl.

        Optional link to an image which represents a type of the resource, for example the logo of Grafana  # noqa: E501

        :param image_url: The image_url of this ExternalUrl.
        :type image_url: str
        """

        self._image_url = image_url

    @property
    def name(self) -> str:
        """Gets the name of this ExternalUrl.

        Human readable name  # noqa: E501

        :return: The name of this ExternalUrl.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """Sets the name of this ExternalUrl.

        Human readable name  # noqa: E501

        :param name: The name of this ExternalUrl.
        :type name: str
        """

        self._name = name

    @property
    def url(self) -> str:
        """Gets the url of this ExternalUrl.

        Link to a resource  # noqa: E501

        :return: The url of this ExternalUrl.
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url: str):
        """Sets the url of this ExternalUrl.

        Link to a resource  # noqa: E501

        :param url: The url of this ExternalUrl.
        :type url: str
        """

        self._url = url



#
#    Copyright 2017 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
API client
"""
import typing

from odahuflow.sdk.clients.api import RemoteAPIClient, AsyncRemoteAPIClient
from odahuflow.sdk.definitions import TOOLCHAIN_INTEGRATION_URL
from odahuflow.sdk.models import ToolchainIntegration


class ToolchainIntegrationClient(RemoteAPIClient):
    """
    HTTP toolchain integration client
    """

    def get(self, name: str) -> ToolchainIntegration:
        """
        Get Toolchain Integration from API server

        :param name: Toolchain Integration name
        :type name: str
        :return: Toolchain Integration
        """
        return ToolchainIntegration.from_dict(self.query(f'{TOOLCHAIN_INTEGRATION_URL}/{name}'))

    def get_all(self) -> typing.List[ToolchainIntegration]:
        """
        Get all Toolchain Integrations from API server

        :return: all Toolchain Integrations
        """
        return [ToolchainIntegration.from_dict(ti) for ti in self.query(TOOLCHAIN_INTEGRATION_URL)]

    def create(self, ti: ToolchainIntegration) -> ToolchainIntegration:
        """
        Create Toolchain Integration

        :param ti: Toolchain Integration
        :return Message from API server
        """
        return ToolchainIntegration.from_dict(
            self.query(TOOLCHAIN_INTEGRATION_URL, action='POST', payload=ti.to_dict())
        )

    def edit(self, ti: ToolchainIntegration) -> ToolchainIntegration:
        """
        Edit Toolchain Integration

        :param ti: Toolchain Integration
        :return Message from API server
        """
        return ToolchainIntegration.from_dict(
            self.query(TOOLCHAIN_INTEGRATION_URL, action='PUT', payload=ti.to_dict())
        )

    def delete(self, name: str) -> str:
        """
        Delete Toolchain Integrations

        :param name: Name of a Toolchain Integration
        :return Message from API server
        """
        return self.query(f'{TOOLCHAIN_INTEGRATION_URL}/{name}', action='DELETE')


class AsyncToolchainIntegrationClient(AsyncRemoteAPIClient):
    """
    HTTP toolchain integration async client
    """
    async def get(self, name: str) -> ToolchainIntegration:
        """
        Get Toolchain Integration from API server

        :param name: Toolchain Integration name
        :type name: str
        :return: Toolchain Integration
        """
        return ToolchainIntegration.from_dict(await self.query(f'{TOOLCHAIN_INTEGRATION_URL}/{name}'))

    async def get_all(self) -> typing.List[ToolchainIntegration]:
        """
        Get all Toolchain Integrations from API server

        :return: all Toolchain Integrations
        """
        return [ToolchainIntegration.from_dict(ti) for ti in await self.query(TOOLCHAIN_INTEGRATION_URL)]

    async def create(self, ti: ToolchainIntegration) -> ToolchainIntegration:
        """
        Create Toolchain Integration

        :param ti: Toolchain Integration
        :return Message from API server
        """
        return ToolchainIntegration.from_dict(
            await self.query(TOOLCHAIN_INTEGRATION_URL, action='POST', payload=ti.to_dict())
        )

    async def edit(self, ti: ToolchainIntegration) -> ToolchainIntegration:
        """
        Edit Toolchain Integration

        :param ti: Toolchain Integration
        :return Message from API server
        """
        return ToolchainIntegration.from_dict(
            await self.query(TOOLCHAIN_INTEGRATION_URL, action='PUT', payload=ti.to_dict())
        )

    async def delete(self, name: str) -> str:
        """
        Delete Toolchain Integrations

        :param name: Name of a Toolchain Integration
        :return Message from API server
        """
        return await self.query(f'{TOOLCHAIN_INTEGRATION_URL}/{name}', action='DELETE')



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class ModelRouteStatus(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, created_at: str=None, edge_url: str=None, state: str=None, updated_at: str=None):  # noqa: E501
        """ModelRouteStatus - a model defined in Swagger

        :param created_at: The created_at of this ModelRouteStatus.  # noqa: E501
        :type created_at: str
        :param edge_url: The edge_url of this ModelRouteStatus.  # noqa: E501
        :type edge_url: str
        :param state: The state of this ModelRouteStatus.  # noqa: E501
        :type state: str
        :param updated_at: The updated_at of this ModelRouteStatus.  # noqa: E501
        :type updated_at: str
        """
        self.swagger_types = {
            'created_at': str,
            'edge_url': str,
            'state': str,
            'updated_at': str
        }

        self.attribute_map = {
            'created_at': 'createdAt',
            'edge_url': 'edgeUrl',
            'state': 'state',
            'updated_at': 'updatedAt'
        }

        self._created_at = created_at
        self._edge_url = edge_url
        self._state = state
        self._updated_at = updated_at

    @classmethod
    def from_dict(cls, dikt) -> 'ModelRouteStatus':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ModelRouteStatus of this ModelRouteStatus.  # noqa: E501
        :rtype: ModelRouteStatus
        """
        return util.deserialize_model(dikt, cls)

    @property
    def created_at(self) -> str:
        """Gets the created_at of this ModelRouteStatus.


        :return: The created_at of this ModelRouteStatus.
        :rtype: str
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at: str):
        """Sets the created_at of this ModelRouteStatus.


        :param created_at: The created_at of this ModelRouteStatus.
        :type created_at: str
        """

        self._created_at = created_at

    @property
    def edge_url(self) -> str:
        """Gets the edge_url of this ModelRouteStatus.

        Full url with prefix to a model deployment service  # noqa: E501

        :return: The edge_url of this ModelRouteStatus.
        :rtype: str
        """
        return self._edge_url

    @edge_url.setter
    def edge_url(self, edge_url: str):
        """Sets the edge_url of this ModelRouteStatus.

        Full url with prefix to a model deployment service  # noqa: E501

        :param edge_url: The edge_url of this ModelRouteStatus.
        :type edge_url: str
        """

        self._edge_url = edge_url

    @property
    def state(self) -> str:
        """Gets the state of this ModelRouteStatus.

        State of Model Route  # noqa: E501

        :return: The state of this ModelRouteStatus.
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state: str):
        """Sets the state of this ModelRouteStatus.

        State of Model Route  # noqa: E501

        :param state: The state of this ModelRouteStatus.
        :type state: str
        """

        self._state = state

    @property
    def updated_at(self) -> str:
        """Gets the updated_at of this ModelRouteStatus.


        :return: The updated_at of this ModelRouteStatus.
        :rtype: str
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at: str):
        """Sets the updated_at of this ModelRouteStatus.


        :param updated_at: The updated_at of this ModelRouteStatus.
        :type updated_at: str
        """

        self._updated_at = updated_at



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.connection import Connection  # noqa: F401,E501
from odahuflow.sdk.models import util


class PackagerTarget(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, connection: Connection=None, name: str=None):  # noqa: E501
        """PackagerTarget - a model defined in Swagger

        :param connection: The connection of this PackagerTarget.  # noqa: E501
        :type connection: Connection
        :param name: The name of this PackagerTarget.  # noqa: E501
        :type name: str
        """
        self.swagger_types = {
            'connection': Connection,
            'name': str
        }

        self.attribute_map = {
            'connection': 'connection',
            'name': 'name'
        }

        self._connection = connection
        self._name = name

    @classmethod
    def from_dict(cls, dikt) -> 'PackagerTarget':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The PackagerTarget of this PackagerTarget.  # noqa: E501
        :rtype: PackagerTarget
        """
        return util.deserialize_model(dikt, cls)

    @property
    def connection(self) -> Connection:
        """Gets the connection of this PackagerTarget.

        A Connection for this target  # noqa: E501

        :return: The connection of this PackagerTarget.
        :rtype: Connection
        """
        return self._connection

    @connection.setter
    def connection(self, connection: Connection):
        """Sets the connection of this PackagerTarget.

        A Connection for this target  # noqa: E501

        :param connection: The connection of this PackagerTarget.
        :type connection: Connection
        """

        self._connection = connection

    @property
    def name(self) -> str:
        """Gets the name of this PackagerTarget.

        Target name  # noqa: E501

        :return: The name of this PackagerTarget.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """Sets the name of this PackagerTarget.

        Target name  # noqa: E501

        :param name: The name of this PackagerTarget.
        :type name: str
        """

        self._name = name



#
#    Copyright 2017 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
legion headers (for meta-information)
"""

REQUEST_ID = 'x-request-id'
MODEL_REQUEST_ID = 'request-id'
MODEL_NAME = 'Model-Name'
MODEL_VERSION = 'Model-Version'



#    Copyright 2019 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

"""
This package must has no requirements except Python3.6
Package provide CLI that can be invoked in any gppi environment without installing extra deps.
The main purpose of package – interact with GPPI folder via CLI in the gppi environment.
This package not provide API to creating and installing Conda Envs by request. You need to create and install
all required GPPI requirements by yourself. (Or check `odahuflowctl gppi` command that can fulfill env requirements
before invoking GPPI)
"""

import argparse
import functools
import importlib
import json
import logging
import os
import pickle
import sys
from contextlib import contextmanager
from typing import Any, List, Union, Dict

_logger = logging.getLogger(__name__)

ODAHUFLOW_MODEL_LOCATION_ENV_VAR = 'MODEL_LOCATION'
MODEL_ENTRYPOINT_ENV = 'MODEL_ENTRYPOINT'

args = None


class CliError(Exception):
    pass


def build_error_response(message):
    raise CliError(message)


@functools.lru_cache()
def get_json_output_serializer():
    entrypoint = get_entrypoint()
    if hasattr(entrypoint, 'get_output_json_serializer'):
        return entrypoint.get_output_json_serializer()
    else:
        return None


def generate_input_props(input_schema: List[Dict[str, Union[Union[str, None, bool], Any]]]) -> Dict[str, Any]:
    """
    Generate input model properties in OpenAPI format
    :param input_schema: Input schema
    :return:
    """
    examples: List[Any] = []
    columns: List[str] = []
    for prop in input_schema:
        columns.append(prop['name'])
        examples.append(prop['example'])

    return {
        "columns": {
            "example": columns,
            "items": {
                "type": "string"
            },
            "type": "array"
        },
        "data": {
            "items": {
                "items": {
                    "type": "number"
                },
                "type": "array"
            },
            "type": "array",
            "example": [examples],
        }
    }


def generate_output_props(output_schema: List[Dict[str, Union[Union[str, None, bool], Any]]]) -> Dict[str, Any]:
    """
    Generate input model properties in OpenAPI format
    :param output_schema:
    :return:
    """
    examples: List[Any] = []
    columns: List[str] = []
    for prop in output_schema:
        columns.append(prop['name'])
        examples.append(prop['example'])

    return {
        "prediction": {
            "example": [examples],
            "items": {
                "type": "number"
            },
            "type": "array",
        },
        "columns": {
            "example": columns,
            "items": {
                "type": "string"
            },
            "type": "array"
        }
    }


def handle_prediction_on_matrix(parsed_data):
    matrix = parsed_data.get('data')
    columns = parsed_data.get('columns', None)

    if not matrix:
        return build_error_response('Matrix is not provided')

    entrypoint = get_entrypoint()

    try:
        prediction, columns = entrypoint.predict_on_matrix(matrix, provided_columns_names=columns)
    except Exception as predict_exception:
        return build_error_response(f'Exception during prediction: {predict_exception}')

    response = {
        'prediction': prediction,
        'columns': columns
    }

    return response


_entrypoint_module = None


def get_entrypoint():
    """
    Return model entrypoint module using required passed argument
    :return:
    """
    if _entrypoint_module is None:
        return importlib.import_module(args.entrypoint)
    else:
        return _entrypoint_module


def get_model_location() -> str:
    """
    Return model location path using environment variable
    :return:
    """
    return os.environ.get(ODAHUFLOW_MODEL_LOCATION_ENV_VAR, ".")


def get_model_input_sample() -> str:
    """
    Return input sample path
    :return:
    """
    return os.path.join(get_model_location(), 'head_input.pkl')


def get_model_output_sample() -> str:
    """
    Return input sample path
    :return:
    """
    return os.path.join(get_model_location(), 'head_output.pkl')


def load_pickle_artifact(name: str) -> Any:
    if os.path.exists(name):
        with open(name, 'rb') as f:
            return pickle.load(f)


def self_check():

    entrypoint = get_entrypoint()

    _logger.info('GPPI entrypoint module successfully imported')

    entrypoint.init()
    _logger.info('GPPI entrypoint.init() – successfully tested')

    entrypoint.info()
    _logger.info('GPPI entrypoint.info() – successfully tested')

    input_ = load_pickle_artifact(get_model_input_sample())
    if input_ is not None:
        entrypoint.predict_on_matrix(input_)
        _logger.info('GPPI entrypoint.predict_on_matrix(...) successfully tested')
    else:
        _logger.warning(f'GPPI {get_model_input_sample()} is not found in model directory. '
                        'Testing of a predicition API is impossible. '
                        'OpenAPI schema will not be generated for the model.')

    print('Self check is successful')


def self_check_caller(args):
    return self_check()


def predict(input_file: str, output_dir: str, output_file_name: str):

    prediction_mode = get_entrypoint().init()

    with open(input_file) as fp:
        parsed_data = json.load(fp)

    if prediction_mode == 'matrix':
        response = handle_prediction_on_matrix(parsed_data)
    else:
        return build_error_response(f'Unknown model\'s return type: {prediction_mode}')

    res_fp = os.path.join(output_dir, output_file_name)
    with open(res_fp, 'w') as fp:
        json.dump(response, fp, cls=get_json_output_serializer())

    print(f'Prediction is successful. Result file: {res_fp}')


def predict_caller(args):
    predict(args.input_file, args.output_dir, args.output_file_name)


def info():
    entrypoint = get_entrypoint()
    input_schema, output_schema = entrypoint.info()
    input_properties = generate_input_props(input_schema)
    output_properties = generate_output_props(output_schema)
    print('Input schema:')
    print(json.dumps(input_properties, indent=4))
    print('Output schema:')
    print(json.dumps(output_properties, indent=4))


def info_caller(args):
    info()


def _configure_logging(verbose: bool):
    if verbose:
        logging.basicConfig(level=logging.DEBUG)


def _configure_arg_parser() -> argparse.ArgumentParser:

    _parser = argparse.ArgumentParser(description="""
    Provide CLI to invoke GPPI entrypoint API.
    This module has only stdlib dependencies so it could be executed from any GPPI environment
    """)
    _parser.add_argument('-v', help='Verbosity logs', action='store_true')
    _parser.add_argument('--entrypoint', help='Name of entrypoint GPPI module')
    _parser.add_argument('--model', help="""
    Override $MODEL_LOCATION environment variable before entrypoint import.
    Clean overridden value after script execution (success or fail).
    """)
    _parser.set_defaults(func=lambda args_: _parser.print_help())

    subparsers = _parser.add_subparsers()

    parser_self_check = subparsers.add_parser('self_check', help='Self check GPPI correctness')
    parser_self_check.set_defaults(func=self_check_caller)

    parser_predict = subparsers.add_parser('predict', help='Make predictions using GPPI model')
    parser_predict.add_argument('input_file', help='JSON file with input data for prediction')
    parser_predict.add_argument('output_dir', help='Directory where JSON with predictions will be saved')
    parser_predict.add_argument('--output_file_name', help='JSON filename with predictions', default='results.json')
    parser_predict.set_defaults(func=predict_caller)

    parser_info = subparsers.add_parser('info', help='Show model input/output data schema')
    parser_info.set_defaults(func=info_caller)

    return _parser


@contextmanager
def model_location(model):
    """
    Set ODAHUFLOW_MODEL_LOCATION_ENV_VAR to `model` if `model` is not None
    Add ODAHUFLOW_MODEL_LOCATION_ENV_VAR to sys.path

    Clean state in context manager exit
    :param model:
    :return:
    """
    model_location_for_use = original_model_location = os.environ.get(ODAHUFLOW_MODEL_LOCATION_ENV_VAR, "")
    if original_model_location:
        _logger.debug(f'{ODAHUFLOW_MODEL_LOCATION_ENV_VAR} env var is set in a system '
                      f'({ODAHUFLOW_MODEL_LOCATION_ENV_VAR}={original_model_location})')
    else:
        _logger.debug(f'{ODAHUFLOW_MODEL_LOCATION_ENV_VAR} env is not set in a system')

    if model:
        _logger.debug(f'--model option is passed. {ODAHUFLOW_MODEL_LOCATION_ENV_VAR} will be replaced '
                      f'with {model}')
        model_location_for_use = os.environ[ODAHUFLOW_MODEL_LOCATION_ENV_VAR] = model

    if not model_location_for_use:
        raise RuntimeError(f'Either {ODAHUFLOW_MODEL_LOCATION_ENV_VAR} env var or --model option MUST be specified')

    sys.path.append(model_location_for_use)
    _logger.debug(f'{model_location_for_use} is added to sys.path')

    try:
        yield
    finally:
        if model:
            os.environ[ODAHUFLOW_MODEL_LOCATION_ENV_VAR] = original_model_location
            _logger.debug(f'{ODAHUFLOW_MODEL_LOCATION_ENV_VAR} is set to original value={original_model_location}')
        if model_location_for_use:
            sys.path.remove(model_location_for_use)
            _logger.debug(f'{model_location_for_use} is removed from sys.path')


def main():

    global args

    parser = _configure_arg_parser()

    args = parser.parse_args()

    if not args.entrypoint:
        args.entrypoint = os.environ.get(MODEL_ENTRYPOINT_ENV)
    if not args.entrypoint:
        raise RuntimeError(f'Either {MODEL_ENTRYPOINT_ENV} env var or --entrypoint option '
                           f'MUST be specified')

    _configure_logging(args.v)

    with model_location(args.model):
        try:
            args.func(args)
        except ImportError as exc_info:
            raise ImportError(
                'ImportError usually happens when you have not packed all required dependencies for you model.'
                'Please see your Training Toolchain documentation to get more info about packing your '
                'model script dependencies'
            ) from exc_info


if __name__ == '__main__':
    main()



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class Target(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, connection_name: str=None, name: str=None):  # noqa: E501
        """Target - a model defined in Swagger

        :param connection_name: The connection_name of this Target.  # noqa: E501
        :type connection_name: str
        :param name: The name of this Target.  # noqa: E501
        :type name: str
        """
        self.swagger_types = {
            'connection_name': str,
            'name': str
        }

        self.attribute_map = {
            'connection_name': 'connectionName',
            'name': 'name'
        }

        self._connection_name = connection_name
        self._name = name

    @classmethod
    def from_dict(cls, dikt) -> 'Target':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Target of this Target.  # noqa: E501
        :rtype: Target
        """
        return util.deserialize_model(dikt, cls)

    @property
    def connection_name(self) -> str:
        """Gets the connection_name of this Target.


        :return: The connection_name of this Target.
        :rtype: str
        """
        return self._connection_name

    @connection_name.setter
    def connection_name(self, connection_name: str):
        """Sets the connection_name of this Target.


        :param connection_name: The connection_name of this Target.
        :type connection_name: str
        """

        self._connection_name = connection_name

    @property
    def name(self) -> str:
        """Gets the name of this Target.


        :return: The name of this Target.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """Sets the name of this Target.


        :param name: The name of this Target.
        :type name: str
        """

        self._name = name



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.resource_requirements import ResourceRequirements  # noqa: F401,E501
from odahuflow.sdk.models.target import Target  # noqa: F401,E501
from odahuflow.sdk.models import util


class ModelPackagingSpec(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, arguments: object=None, artifact_name: str=None, image: str=None, integration_name: str=None, output_connection: str=None, resources: ResourceRequirements=None, targets: List[Target]=None):  # noqa: E501
        """ModelPackagingSpec - a model defined in Swagger

        :param arguments: The arguments of this ModelPackagingSpec.  # noqa: E501
        :type arguments: object
        :param artifact_name: The artifact_name of this ModelPackagingSpec.  # noqa: E501
        :type artifact_name: str
        :param image: The image of this ModelPackagingSpec.  # noqa: E501
        :type image: str
        :param integration_name: The integration_name of this ModelPackagingSpec.  # noqa: E501
        :type integration_name: str
        :param output_connection: The output_connection of this ModelPackagingSpec.  # noqa: E501
        :type output_connection: str
        :param resources: The resources of this ModelPackagingSpec.  # noqa: E501
        :type resources: ResourceRequirements
        :param targets: The targets of this ModelPackagingSpec.  # noqa: E501
        :type targets: List[Target]
        """
        self.swagger_types = {
            'arguments': object,
            'artifact_name': str,
            'image': str,
            'integration_name': str,
            'output_connection': str,
            'resources': ResourceRequirements,
            'targets': List[Target]
        }

        self.attribute_map = {
            'arguments': 'arguments',
            'artifact_name': 'artifactName',
            'image': 'image',
            'integration_name': 'integrationName',
            'output_connection': 'outputConnection',
            'resources': 'resources',
            'targets': 'targets'
        }

        self._arguments = arguments
        self._artifact_name = artifact_name
        self._image = image
        self._integration_name = integration_name
        self._output_connection = output_connection
        self._resources = resources
        self._targets = targets

    @classmethod
    def from_dict(cls, dikt) -> 'ModelPackagingSpec':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ModelPackagingSpec of this ModelPackagingSpec.  # noqa: E501
        :rtype: ModelPackagingSpec
        """
        return util.deserialize_model(dikt, cls)

    @property
    def arguments(self) -> object:
        """Gets the arguments of this ModelPackagingSpec.

        List of arguments. This parameter depends on the specific packaging integration  # noqa: E501

        :return: The arguments of this ModelPackagingSpec.
        :rtype: object
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments: object):
        """Sets the arguments of this ModelPackagingSpec.

        List of arguments. This parameter depends on the specific packaging integration  # noqa: E501

        :param arguments: The arguments of this ModelPackagingSpec.
        :type arguments: object
        """

        self._arguments = arguments

    @property
    def artifact_name(self) -> str:
        """Gets the artifact_name of this ModelPackagingSpec.

        Training output artifact name  # noqa: E501

        :return: The artifact_name of this ModelPackagingSpec.
        :rtype: str
        """
        return self._artifact_name

    @artifact_name.setter
    def artifact_name(self, artifact_name: str):
        """Sets the artifact_name of this ModelPackagingSpec.

        Training output artifact name  # noqa: E501

        :param artifact_name: The artifact_name of this ModelPackagingSpec.
        :type artifact_name: str
        """

        self._artifact_name = artifact_name

    @property
    def image(self) -> str:
        """Gets the image of this ModelPackagingSpec.

        Image name. Packaging integration image will be used if this parameters is missed  # noqa: E501

        :return: The image of this ModelPackagingSpec.
        :rtype: str
        """
        return self._image

    @image.setter
    def image(self, image: str):
        """Sets the image of this ModelPackagingSpec.

        Image name. Packaging integration image will be used if this parameters is missed  # noqa: E501

        :param image: The image of this ModelPackagingSpec.
        :type image: str
        """

        self._image = image

    @property
    def integration_name(self) -> str:
        """Gets the integration_name of this ModelPackagingSpec.

        Packaging integration ID  # noqa: E501

        :return: The integration_name of this ModelPackagingSpec.
        :rtype: str
        """
        return self._integration_name

    @integration_name.setter
    def integration_name(self, integration_name: str):
        """Sets the integration_name of this ModelPackagingSpec.

        Packaging integration ID  # noqa: E501

        :param integration_name: The integration_name of this ModelPackagingSpec.
        :type integration_name: str
        """

        self._integration_name = integration_name

    @property
    def output_connection(self) -> str:
        """Gets the output_connection of this ModelPackagingSpec.

        Name of Connection to storage where a packager obtain a model trained artifact. Permitted connection types are defined by specific PackagingIntegration  # noqa: E501

        :return: The output_connection of this ModelPackagingSpec.
        :rtype: str
        """
        return self._output_connection

    @output_connection.setter
    def output_connection(self, output_connection: str):
        """Sets the output_connection of this ModelPackagingSpec.

        Name of Connection to storage where a packager obtain a model trained artifact. Permitted connection types are defined by specific PackagingIntegration  # noqa: E501

        :param output_connection: The output_connection of this ModelPackagingSpec.
        :type output_connection: str
        """

        self._output_connection = output_connection

    @property
    def resources(self) -> ResourceRequirements:
        """Gets the resources of this ModelPackagingSpec.

        Resources for packager container The same format like k8s uses for pod resources.  # noqa: E501

        :return: The resources of this ModelPackagingSpec.
        :rtype: ResourceRequirements
        """
        return self._resources

    @resources.setter
    def resources(self, resources: ResourceRequirements):
        """Sets the resources of this ModelPackagingSpec.

        Resources for packager container The same format like k8s uses for pod resources.  # noqa: E501

        :param resources: The resources of this ModelPackagingSpec.
        :type resources: ResourceRequirements
        """

        self._resources = resources

    @property
    def targets(self) -> List[Target]:
        """Gets the targets of this ModelPackagingSpec.

        List of targets. This parameter depends on the specific packaging integration  # noqa: E501

        :return: The targets of this ModelPackagingSpec.
        :rtype: List[Target]
        """
        return self._targets

    @targets.setter
    def targets(self, targets: List[Target]):
        """Sets the targets of this ModelPackagingSpec.

        List of targets. This parameter depends on the specific packaging integration  # noqa: E501

        :param targets: The targets of this ModelPackagingSpec.
        :type targets: List[Target]
        """

        self._targets = targets



#
#    Copyright 2017 EPAM Systems
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
"""
API client
"""
import logging

from odahuflow.sdk.clients.api import RemoteAPIClient, AsyncRemoteAPIClient
from odahuflow.sdk.definitions import CONFIGURATION_URL
from odahuflow.sdk.models import Configuration

LOGGER = logging.getLogger(__name__)


class ConfigurationClient(RemoteAPIClient):
    """
    HTTP connection client
    """

    def get(self) -> Configuration:
        """
        Get Configuration from API server

        :return: Configuration
        """
        return Configuration.from_dict(self.query(CONFIGURATION_URL))

    def edit(self, conf: Configuration) -> Configuration:
        """
        Edit Configuration

        :param conf: Configuration
        :return Message from API server
        """
        return Configuration.from_dict(self.query(CONFIGURATION_URL, action='PUT', payload=conf.to_dict()))


class AsyncConfigurationClient(AsyncRemoteAPIClient):
    """
    HTTP connection async client
    """

    async def get(self) -> Configuration:
        """
        Get Configuration from API server

        :return: Configuration
        """
        return Configuration.from_dict(await self.query(CONFIGURATION_URL))

    async def edit(self, conf: Configuration) -> Configuration:
        """
        Edit Configuration

        :param conf: Configuration
        :return Message from API server
        """
        return Configuration.from_dict(await self.query(CONFIGURATION_URL, action='PUT', payload=conf.to_dict()))



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class EnvironmentVariable(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, name: str=None, value: str=None):  # noqa: E501
        """EnvironmentVariable - a model defined in Swagger

        :param name: The name of this EnvironmentVariable.  # noqa: E501
        :type name: str
        :param value: The value of this EnvironmentVariable.  # noqa: E501
        :type value: str
        """
        self.swagger_types = {
            'name': str,
            'value': str
        }

        self.attribute_map = {
            'name': 'name',
            'value': 'value'
        }

        self._name = name
        self._value = value

    @classmethod
    def from_dict(cls, dikt) -> 'EnvironmentVariable':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The EnvironmentVariable of this EnvironmentVariable.  # noqa: E501
        :rtype: EnvironmentVariable
        """
        return util.deserialize_model(dikt, cls)

    @property
    def name(self) -> str:
        """Gets the name of this EnvironmentVariable.

        Name of an environment variable  # noqa: E501

        :return: The name of this EnvironmentVariable.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """Sets the name of this EnvironmentVariable.

        Name of an environment variable  # noqa: E501

        :param name: The name of this EnvironmentVariable.
        :type name: str
        """

        self._name = name

    @property
    def value(self) -> str:
        """Gets the value of this EnvironmentVariable.

        Value of an environment variable  # noqa: E501

        :return: The value of this EnvironmentVariable.
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value: str):
        """Sets the value of this EnvironmentVariable.

        Value of an environment variable  # noqa: E501

        :param value: The value of this EnvironmentVariable.
        :type value: str
        """

        self._value = value



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class ResourceList(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, cpu: str=None, gpu: str=None, memory: str=None):  # noqa: E501
        """ResourceList - a model defined in Swagger

        :param cpu: The cpu of this ResourceList.  # noqa: E501
        :type cpu: str
        :param gpu: The gpu of this ResourceList.  # noqa: E501
        :type gpu: str
        :param memory: The memory of this ResourceList.  # noqa: E501
        :type memory: str
        """
        self.swagger_types = {
            'cpu': str,
            'gpu': str,
            'memory': str
        }

        self.attribute_map = {
            'cpu': 'cpu',
            'gpu': 'gpu',
            'memory': 'memory'
        }

        self._cpu = cpu
        self._gpu = gpu
        self._memory = memory

    @classmethod
    def from_dict(cls, dikt) -> 'ResourceList':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ResourceList of this ResourceList.  # noqa: E501
        :rtype: ResourceList
        """
        return util.deserialize_model(dikt, cls)

    @property
    def cpu(self) -> str:
        """Gets the cpu of this ResourceList.

        Read more about CPU resource here https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/#meaning-of-cpu  # noqa: E501

        :return: The cpu of this ResourceList.
        :rtype: str
        """
        return self._cpu

    @cpu.setter
    def cpu(self, cpu: str):
        """Sets the cpu of this ResourceList.

        Read more about CPU resource here https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/#meaning-of-cpu  # noqa: E501

        :param cpu: The cpu of this ResourceList.
        :type cpu: str
        """

        self._cpu = cpu

    @property
    def gpu(self) -> str:
        """Gets the gpu of this ResourceList.

        Read more about GPU resource here https://kubernetes.io/docs/tasks/manage-gpus/scheduling-gpus/#using-device-plugins  # noqa: E501

        :return: The gpu of this ResourceList.
        :rtype: str
        """
        return self._gpu

    @gpu.setter
    def gpu(self, gpu: str):
        """Sets the gpu of this ResourceList.

        Read more about GPU resource here https://kubernetes.io/docs/tasks/manage-gpus/scheduling-gpus/#using-device-plugins  # noqa: E501

        :param gpu: The gpu of this ResourceList.
        :type gpu: str
        """

        self._gpu = gpu

    @property
    def memory(self) -> str:
        """Gets the memory of this ResourceList.

        Read more about memory resource here https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/#meaning-of-memory  # noqa: E501

        :return: The memory of this ResourceList.
        :rtype: str
        """
        return self._memory

    @memory.setter
    def memory(self, memory: str):
        """Sets the memory of this ResourceList.

        Read more about memory resource here https://kubernetes.io/docs/concepts/configuration/manage-compute-resources-container/#meaning-of-memory  # noqa: E501

        :param memory: The memory of this ResourceList.
        :type memory: str
        """

        self._memory = memory



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models import util


class TrainingResult(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, artifact_name: str=None, commit_id: str=None, run_id: str=None):  # noqa: E501
        """TrainingResult - a model defined in Swagger

        :param artifact_name: The artifact_name of this TrainingResult.  # noqa: E501
        :type artifact_name: str
        :param commit_id: The commit_id of this TrainingResult.  # noqa: E501
        :type commit_id: str
        :param run_id: The run_id of this TrainingResult.  # noqa: E501
        :type run_id: str
        """
        self.swagger_types = {
            'artifact_name': str,
            'commit_id': str,
            'run_id': str
        }

        self.attribute_map = {
            'artifact_name': 'artifactName',
            'commit_id': 'commitID',
            'run_id': 'runId'
        }

        self._artifact_name = artifact_name
        self._commit_id = commit_id
        self._run_id = run_id

    @classmethod
    def from_dict(cls, dikt) -> 'TrainingResult':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The TrainingResult of this TrainingResult.  # noqa: E501
        :rtype: TrainingResult
        """
        return util.deserialize_model(dikt, cls)

    @property
    def artifact_name(self) -> str:
        """Gets the artifact_name of this TrainingResult.

        Trained artifact name  # noqa: E501

        :return: The artifact_name of this TrainingResult.
        :rtype: str
        """
        return self._artifact_name

    @artifact_name.setter
    def artifact_name(self, artifact_name: str):
        """Sets the artifact_name of this TrainingResult.

        Trained artifact name  # noqa: E501

        :param artifact_name: The artifact_name of this TrainingResult.
        :type artifact_name: str
        """

        self._artifact_name = artifact_name

    @property
    def commit_id(self) -> str:
        """Gets the commit_id of this TrainingResult.

        VCS commit  # noqa: E501

        :return: The commit_id of this TrainingResult.
        :rtype: str
        """
        return self._commit_id

    @commit_id.setter
    def commit_id(self, commit_id: str):
        """Sets the commit_id of this TrainingResult.

        VCS commit  # noqa: E501

        :param commit_id: The commit_id of this TrainingResult.
        :type commit_id: str
        """

        self._commit_id = commit_id

    @property
    def run_id(self) -> str:
        """Gets the run_id of this TrainingResult.

        Mlflow run ID  # noqa: E501

        :return: The run_id of this TrainingResult.
        :rtype: str
        """
        return self._run_id

    @run_id.setter
    def run_id(self, run_id: str):
        """Sets the run_id of this TrainingResult.

        Mlflow run ID  # noqa: E501

        :param run_id: The run_id of this TrainingResult.
        :type run_id: str
        """

        self._run_id = run_id



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.model_property import ModelProperty  # noqa: F401,E501
from odahuflow.sdk.models import util


class JsonSchema(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, properties: List[ModelProperty]=None, required: List[str]=None):  # noqa: E501
        """JsonSchema - a model defined in Swagger

        :param properties: The properties of this JsonSchema.  # noqa: E501
        :type properties: List[ModelProperty]
        :param required: The required of this JsonSchema.  # noqa: E501
        :type required: List[str]
        """
        self.swagger_types = {
            'properties': List[ModelProperty],
            'required': List[str]
        }

        self.attribute_map = {
            'properties': 'properties',
            'required': 'required'
        }

        self._properties = properties
        self._required = required

    @classmethod
    def from_dict(cls, dikt) -> 'JsonSchema':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The JsonSchema of this JsonSchema.  # noqa: E501
        :rtype: JsonSchema
        """
        return util.deserialize_model(dikt, cls)

    @property
    def properties(self) -> List[ModelProperty]:
        """Gets the properties of this JsonSchema.

        Properties configuration  # noqa: E501

        :return: The properties of this JsonSchema.
        :rtype: List[ModelProperty]
        """
        return self._properties

    @properties.setter
    def properties(self, properties: List[ModelProperty]):
        """Sets the properties of this JsonSchema.

        Properties configuration  # noqa: E501

        :param properties: The properties of this JsonSchema.
        :type properties: List[ModelProperty]
        """

        self._properties = properties

    @property
    def required(self) -> List[str]:
        """Gets the required of this JsonSchema.

        List of required properties  # noqa: E501

        :return: The required of this JsonSchema.
        :rtype: List[str]
        """
        return self._required

    @required.setter
    def required(self, required: List[str]):
        """Sets the required of this JsonSchema.

        List of required properties  # noqa: E501

        :param required: The required of this JsonSchema.
        :type required: List[str]
        """

        self._required = required



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.model_deployment_target import ModelDeploymentTarget  # noqa: F401,E501
from odahuflow.sdk.models import util


class ModelRouteSpec(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, mirror: str=None, model_deployments: List[ModelDeploymentTarget]=None, url_prefix: str=None):  # noqa: E501
        """ModelRouteSpec - a model defined in Swagger

        :param mirror: The mirror of this ModelRouteSpec.  # noqa: E501
        :type mirror: str
        :param model_deployments: The model_deployments of this ModelRouteSpec.  # noqa: E501
        :type model_deployments: List[ModelDeploymentTarget]
        :param url_prefix: The url_prefix of this ModelRouteSpec.  # noqa: E501
        :type url_prefix: str
        """
        self.swagger_types = {
            'mirror': str,
            'model_deployments': List[ModelDeploymentTarget],
            'url_prefix': str
        }

        self.attribute_map = {
            'mirror': 'mirror',
            'model_deployments': 'modelDeployments',
            'url_prefix': 'urlPrefix'
        }

        self._mirror = mirror
        self._model_deployments = model_deployments
        self._url_prefix = url_prefix

    @classmethod
    def from_dict(cls, dikt) -> 'ModelRouteSpec':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The ModelRouteSpec of this ModelRouteSpec.  # noqa: E501
        :rtype: ModelRouteSpec
        """
        return util.deserialize_model(dikt, cls)

    @property
    def mirror(self) -> str:
        """Gets the mirror of this ModelRouteSpec.

        Mirror HTTP traffic to a another Model deployment in addition to forwarding the requests to the model deployments.  # noqa: E501

        :return: The mirror of this ModelRouteSpec.
        :rtype: str
        """
        return self._mirror

    @mirror.setter
    def mirror(self, mirror: str):
        """Sets the mirror of this ModelRouteSpec.

        Mirror HTTP traffic to a another Model deployment in addition to forwarding the requests to the model deployments.  # noqa: E501

        :param mirror: The mirror of this ModelRouteSpec.
        :type mirror: str
        """

        self._mirror = mirror

    @property
    def model_deployments(self) -> List[ModelDeploymentTarget]:
        """Gets the model_deployments of this ModelRouteSpec.

        A http rule can forward traffic to Model Deployments.  # noqa: E501

        :return: The model_deployments of this ModelRouteSpec.
        :rtype: List[ModelDeploymentTarget]
        """
        return self._model_deployments

    @model_deployments.setter
    def model_deployments(self, model_deployments: List[ModelDeploymentTarget]):
        """Sets the model_deployments of this ModelRouteSpec.

        A http rule can forward traffic to Model Deployments.  # noqa: E501

        :param model_deployments: The model_deployments of this ModelRouteSpec.
        :type model_deployments: List[ModelDeploymentTarget]
        """

        self._model_deployments = model_deployments

    @property
    def url_prefix(self) -> str:
        """Gets the url_prefix of this ModelRouteSpec.

        URL prefix for model  For example: /custom/test Prefix must start with slash \"/feedback\" and \"/model\" are reserved for internal usage  # noqa: E501

        :return: The url_prefix of this ModelRouteSpec.
        :rtype: str
        """
        return self._url_prefix

    @url_prefix.setter
    def url_prefix(self, url_prefix: str):
        """Sets the url_prefix of this ModelRouteSpec.

        URL prefix for model  For example: /custom/test Prefix must start with slash \"/feedback\" and \"/model\" are reserved for internal usage  # noqa: E501

        :param url_prefix: The url_prefix of this ModelRouteSpec.
        :type url_prefix: str
        """

        self._url_prefix = url_prefix



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from odahuflow.sdk.models.base_model_ import Model
from odahuflow.sdk.models.schema import Schema  # noqa: F401,E501
from odahuflow.sdk.models import util


class PackagingIntegrationSpec(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, default_image: str=None, entrypoint: str=None, privileged: bool=None, schema: Schema=None):  # noqa: E501
        """PackagingIntegrationSpec - a model defined in Swagger

        :param default_image: The default_image of this PackagingIntegrationSpec.  # noqa: E501
        :type default_image: str
        :param entrypoint: The entrypoint of this PackagingIntegrationSpec.  # noqa: E501
        :type entrypoint: str
        :param privileged: The privileged of this PackagingIntegrationSpec.  # noqa: E501
        :type privileged: bool
        :param schema: The schema of this PackagingIntegrationSpec.  # noqa: E501
        :type schema: Schema
        """
        self.swagger_types = {
            'default_image': str,
            'entrypoint': str,
            'privileged': bool,
            'schema': Schema
        }

        self.attribute_map = {
            'default_image': 'defaultImage',
            'entrypoint': 'entrypoint',
            'privileged': 'privileged',
            'schema': 'schema'
        }

        self._default_image = default_image
        self._entrypoint = entrypoint
        self._privileged = privileged
        self._schema = schema

    @classmethod
    def from_dict(cls, dikt) -> 'PackagingIntegrationSpec':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The PackagingIntegrationSpec of this PackagingIntegrationSpec.  # noqa: E501
        :rtype: PackagingIntegrationSpec
        """
        return util.deserialize_model(dikt, cls)

    @property
    def default_image(self) -> str:
        """Gets the default_image of this PackagingIntegrationSpec.

        Default packaging Docker image  # noqa: E501

        :return: The default_image of this PackagingIntegrationSpec.
        :rtype: str
        """
        return self._default_image

    @default_image.setter
    def default_image(self, default_image: str):
        """Sets the default_image of this PackagingIntegrationSpec.

        Default packaging Docker image  # noqa: E501

        :param default_image: The default_image of this PackagingIntegrationSpec.
        :type default_image: str
        """

        self._default_image = default_image

    @property
    def entrypoint(self) -> str:
        """Gets the entrypoint of this PackagingIntegrationSpec.

        Path to binary which starts a packaging process  # noqa: E501

        :return: The entrypoint of this PackagingIntegrationSpec.
        :rtype: str
        """
        return self._entrypoint

    @entrypoint.setter
    def entrypoint(self, entrypoint: str):
        """Sets the entrypoint of this PackagingIntegrationSpec.

        Path to binary which starts a packaging process  # noqa: E501

        :param entrypoint: The entrypoint of this PackagingIntegrationSpec.
        :type entrypoint: str
        """

        self._entrypoint = entrypoint

    @property
    def privileged(self) -> bool:
        """Gets the privileged of this PackagingIntegrationSpec.

        Enable docker privileged flag  # noqa: E501

        :return: The privileged of this PackagingIntegrationSpec.
        :rtype: bool
        """
        return self._privileged

    @privileged.setter
    def privileged(self, privileged: bool):
        """Sets the privileged of this PackagingIntegrationSpec.

        Enable docker privileged flag  # noqa: E501

        :param privileged: The privileged of this PackagingIntegrationSpec.
        :type privileged: bool
        """

        self._privileged = privileged

    @property
    def schema(self) -> Schema:
        """Gets the schema of this PackagingIntegrationSpec.

        Schema which describes targets and arguments for specific packaging integration  # noqa: E501

        :return: The schema of this PackagingIntegrationSpec.
        :rtype: Schema
        """
        return self._schema

    @schema.setter
    def schema(self, schema: Schema):
        """Sets the schema of this PackagingIntegrationSpec.

        Schema which describes targets and arguments for specific packaging integration  # noqa: E501

        :param schema: The schema of this PackagingIntegrationSpec.
        :type schema: Schema
        """

        self._schema = schema
